@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -52,7 +52,7 @@
 #include "lislib.h"
 
 /***************************************
- * Laczos Iteration                    *
+ * Lanczos Iteration                   *
  ***************************************
  v(0)    = (0,...,0)^T
  ***************************************
@@ -61,23 +61,23 @@
  for j=1,2,...
    v(j)      = r / beta(j-1)
    r         = A * v(j)
-   r         = r - v(j-1) * beta(j-1)
-   alpha(j)  = v(j) * r
-   r         = r - v(j) * alpha(j)
+   r         = r - beta(j-1) * v(j-1)
+   alpha(j)  = <v(j), r>
+   r         = r - alpha(j) * v(j)
    reorthogonalization 
    beta(j)   = ||r||_2
-   compute approximate eigenvalues of a real symmetric 
-   tridiagonal matrix T(j) = ST'(j)S^*, where
-       alpha(1) beta(1) 
-       beta(1)  alpha(2) 
-   T =                ...
-                         alpha(j-1) beta(j-1)                        
-                         beta(j-1)  alpha(j)
  end for
- compute approximate eigenvectors X = V(j)S
+ compute eigenvalues of a real symmetric tridiagonal matrix 
+ T(j) = ST'(j)S^*, where
+     (alpha(1) beta(1)                      )
+     (beta(1)  alpha(2)                     )
+ T = (               ...                    )
+     (                  alpha(j-1) beta(j-1))                       
+     (                  beta(j-1)  alpha(j) )
+ compute refined eigenpairs
  ***************************************/
 
-#define NWORK 3
+#define NWORK 2
 #undef __FUNC__
 #define __FUNC__ "lis_eli_check_params"
 LIS_INT lis_eli_check_params(LIS_ESOLVER esolver)
@@ -101,8 +101,8 @@ LIS_INT lis_eli_check_params(LIS_ESOLVER esolver)
 #define __FUNC__ "lis_eli_malloc_work"
 LIS_INT lis_eli_malloc_work(LIS_ESOLVER esolver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err,ss;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err,ss;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -148,22 +148,20 @@ LIS_INT lis_eli_malloc_work(LIS_ESOLVER esolver)
 #define __FUNC__ "lis_eli"
 LIS_INT lis_eli(LIS_ESOLVER esolver)
 {
-  LIS_MATRIX        A;
-  LIS_VECTOR        x;
-  LIS_SCALAR        lshift;
-  LIS_INT               ss;
-  LIS_INT               emaxiter,iter0;
-  LIS_REAL          tol;
-  LIS_INT               i,j,k;
-  LIS_INT               output, niesolver;
-  LIS_REAL          nrm2,dot,resid,resid0,*residual0;
-  LIS_VECTOR        *v,r;
-  LIS_SCALAR        *t, *tx, tevalue, *tq, *tr, evalue, evalue0;
-  LIS_SOLVER        solver;
-  LIS_PRECON        precon;
-  LIS_ESOLVER       esolver2;
-  char              esolvername[128], solvername[128], preconname[128];
-  LIS_INT           nsol, precon_type;
+  LIS_MATRIX A;
+  LIS_INT ss,ic;
+  LIS_INT emaxiter,iter0,qriter;
+  LIS_REAL tol,qrerr;
+  LIS_INT i,j,k;
+  LIS_INT output, niesolver;
+  LIS_REAL nrm2,resid0;
+  LIS_SCALAR dot;
+  LIS_VECTOR *v,r;
+  LIS_SCALAR *t,*tq,*tr,evalue,evalue0;
+  LIS_SOLVER solver;
+  LIS_ESOLVER esolver2;
+  char esolvername[128],solvername[128],preconname[128];
+  LIS_INT nsol,precon_type;
 
   ss = esolver->options[LIS_EOPTIONS_SUBSPACE];
   emaxiter = esolver->options[LIS_EOPTIONS_MAXITER];
@@ -172,35 +170,27 @@ LIS_INT lis_eli(LIS_ESOLVER esolver)
   niesolver = esolver->options[LIS_EOPTIONS_INNER_ESOLVER];
 
   t = (LIS_SCALAR *)lis_malloc(ss*ss*sizeof(LIS_SCALAR), "lis_eli::t");
-  tx = (LIS_SCALAR *)lis_malloc(ss*sizeof(LIS_SCALAR), "lis_eli::tx");
   tq = (LIS_SCALAR *)lis_malloc(ss*ss*sizeof(LIS_SCALAR), "lis_eli::tq");
   tr = (LIS_SCALAR *)lis_malloc(ss*ss*sizeof(LIS_SCALAR), "lis_eli::tr");
   
   A = esolver->A;
-  x = esolver->work[0];
-  r = esolver->work[1];
-  v = &esolver->work[2];
+  r = esolver->work[0];
+  v = &esolver->work[1];
   lis_vector_set_all(0.0,v[0]);
   lis_vector_set_all(1.0,r);
   lis_vector_nrm2(r, &nrm2);
 
   lis_solver_create(&solver);
-  lis_solver_set_option("-i cg -p ilu",solver);
+  lis_solver_set_option("-i bicg -p none",solver);
   lis_solver_set_optionC(solver);
   lis_solver_get_solver(solver, &nsol);
   lis_solver_get_precon(solver, &precon_type);
   lis_solver_get_solvername(nsol, solvername);
   lis_solver_get_preconname(precon_type, preconname);
   lis_esolver_get_esolvername(niesolver, esolvername);
-#ifdef _LONGLONG
-  if( A->my_rank==0 ) printf("iesolver   : %s %lld\n", esolvername, niesolver);
-  if( A->my_rank==0 ) printf("solver     : %s %lld\n", solvername, nsol);
-  if( A->my_rank==0 ) printf("precon     : %s %lld\n", preconname, precon_type);
-#else
-  if( A->my_rank==0 ) printf("iesolver   : %s %d\n", esolvername, niesolver);
-  if( A->my_rank==0 ) printf("solver     : %s %d\n", solvername, nsol);
-  if( A->my_rank==0 ) printf("precon     : %s %d\n", preconname, precon_type);
-#endif
+  if( A->my_rank==0 ) printf("inner eigensolver     : %s\n", esolvername);
+  if( A->my_rank==0 ) printf("linear solver         : %s\n", solvername);
+  if( A->my_rank==0 ) printf("preconditioner        : %s\n", preconname);
 
   for (i=0;i<ss*ss;i++) t[i] = 0.0;
 
@@ -208,11 +198,11 @@ LIS_INT lis_eli(LIS_ESOLVER esolver)
   while (j<ss-1)
     {
       j = j+1;
-
-      /* v(j) = r / beta(j-1) */
       lis_vector_copy(r, v[j]);
 
+      /* v(j) = r / beta(j-1) */
       /* r = A * v(j) */
+      /* r = r - beta(j-1) * v(j-1) */
       if (j==1) {
 	lis_vector_scale(1/nrm2, v[j]);
 	lis_matvec(A, v[j], r);
@@ -223,10 +213,10 @@ LIS_INT lis_eli(LIS_ESOLVER esolver)
 	lis_vector_axpy(-t[(j-2)*ss+j-1], v[j-1], r); 
       }
 
-      /* alpha(j)  = v(j) * r */
+      /* alpha(j)  = <v(j), r> */
       lis_vector_dot(v[j], r, &t[(j-1)*ss+j-1]); 
 
-      /* r = r - v(j) * alpha(j) */
+      /* r = r - alpha(j) * v(j) */
       lis_vector_axpy(-t[(j-1)*ss+j-1], v[j], r); 
 
       /* reorthogonalization */
@@ -240,51 +230,60 @@ LIS_INT lis_eli(LIS_ESOLVER esolver)
       lis_vector_nrm2(r, &t[(j-1)*ss+j]);
 
       /* convergence check */
-      if (t[(j-1)*ss+j]<tol) break;  
+      if (fabs(t[(j-1)*ss+j])<tol) break;  
       t[j*ss+j-1] = t[(j-1)*ss+j];
     }
 
-  /* compute approximate eigenvalues of a real symmetric
-     tridiagonal matrix T(j) = ST'(j)S^* */
-  lis_array_qr(ss, t, tq, tr); 
+  /* compute eigenvalues of a real symmetric tridiagonal matrix 
+     T(j) = ST'(j)S^* */
+  lis_array_qr(ss,t,tq,tr,&qriter,&qrerr); 
 
   for (i=0;i<ss;i++)
     {
       esolver->evalue[i] = t[i*ss+i];
     }
-  lis_sort_d(0, ss-1, esolver->evalue);
 
   if( A->my_rank==0 ) 
     {
-#ifdef _LONGLONG
-      printf("size of subspace = %lld\n", ss);
+#ifdef _LONG__LONG
+      printf("size of subspace      : %lld\n\n", ss);
 #else
-      printf("size of subspace = %d\n", ss);
+      printf("size of subspace      : %d\n\n", ss);
 #endif
-      if( output & LIS_PRINT_OUT ) printf("solutions in subspace:\n");
+      if( output ) printf("approximate eigenvalues in subspace:\n\n");
       for (i=0;i<ss;i++)
 	{
-#ifdef _LONGLONG
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %lld\n", i);
+#ifdef _LONG__LONG
+	  if( output ) printf("Lanczos: mode number              = %lld\n", i);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %d\n", i);
+	  if( output ) printf("Lanczos: mode number              = %d\n", i);
 #endif
-#ifdef _LONG_DOUBLE
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %Le\n", esolver->evalue[i]);
+#ifdef _COMPLEX	  
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue               = %Le + %Le * I\n", creall(esolver->evalue[i]), cimagl(esolver->evalue[i]));
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %e\n", esolver->evalue[i]);
+	  if( output ) printf("Lanczos: eigenvalue               = %e + %e * I\n", creal(esolver->evalue[i]), cimag(esolver->evalue[i]));
 #endif
+#else
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue               = %Le\n", esolver->evalue[i]);
+#else
+	  if( output ) printf("Lanczos: eigenvalue               = %e\n", esolver->evalue[i]);
+#endif
+#endif	  
 	}
-      if( output & LIS_PRINT_OUT ) printf("refined solutions:\n");
+      if( output ) printf("\n");
+      if( output ) printf("compute refined eigenpairs:\n\n");
     }
 
   lis_esolver_create(&esolver2);
   esolver2->options[LIS_EOPTIONS_ESOLVER] = niesolver;
   esolver2->options[LIS_EOPTIONS_SUBSPACE] = 1;
   esolver2->options[LIS_EOPTIONS_MAXITER] = emaxiter;
+  esolver2->options[LIS_EOPTIONS_OUTPUT] = esolver->options[LIS_EOPTIONS_OUTPUT];
   esolver2->params[LIS_EPARAMS_RESID - LIS_EOPTIONS_LEN] = tol; 
 
-  /* compute approximate eigenvectors X = V(j)S */
+  /* compute refined eigenpairs */
   for (i=0;i<ss;i++)
     {
       lis_vector_duplicate(A, &esolver->evector[i]); 
@@ -292,54 +291,77 @@ LIS_INT lis_eli(LIS_ESOLVER esolver)
       lis_esolve(A, esolver->evector[i], &evalue, esolver2);
       lis_esolver_work_destroy(esolver2); 
       esolver->evalue[i] = evalue - esolver2->lshift;
+      esolver->iter[i] = esolver2->iter[0];      
+      esolver->resid[i] = esolver2->resid[0];
 
       if (i==0) 
 	{
 	  evalue0 = esolver->evalue[0];
-	  iter0 = esolver2->iter;
-	  resid0 = esolver2->resid;
-	  residual0 = esolver2->residual;
-	  esolver->ptimes = esolver2->ptimes;
-	  esolver->itimes = esolver2->itimes;
-	  esolver->p_c_times = esolver2->p_c_times;
-	  esolver->p_i_times = esolver2->p_i_times;
+	  iter0 = esolver2->iter[0];
+	  resid0 = esolver2->resid[0];
+	  if( output & LIS_EPRINT_MEM ) 
+	    {
+	      for (ic=0;ic<iter0+1;ic++)
+		{
+		  esolver->rhistory[ic] = esolver2->rhistory[ic]; 
+		}
+	    }
+	  esolver->ptime = esolver2->ptime;
+	  esolver->itime = esolver2->itime;
+	  esolver->p_c_time = esolver2->p_c_time;
+	  esolver->p_i_time = esolver2->p_i_time;
 	}
 
       if (A->my_rank==0) 
 	{
 
-#ifdef _LONGLONG
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %lld\n", i);
+#ifdef _LONG__LONG
+	  if( output ) printf("Lanczos: mode number          = %lld\n", i);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %d\n", i);
+	  if( output ) printf("Lanczos: mode number          = %d\n", i);
 #endif
-#ifdef _LONG_DOUBLE
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %Le\n", esolver->evalue[i]);
+#ifdef _COMPLEX
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue           = %Le + %Le * I\n", creall(esolver->evalue[i]), cimagl(esolver->evalue[i]));
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %e\n", esolver->evalue[i]);
+	  if( output ) printf("Lanczos: eigenvalue           = %e + %e * I\n", creal(esolver->evalue[i]), cimag(esolver->evalue[i]));
 #endif
-#ifdef _LONGLONG
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: number of iterations     = %lld\n",esolver2->iter);
+#else	  
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue           = %Le\n", esolver->evalue[i]);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: number of iterations     = %d\n",esolver2->iter);
+	  if( output ) printf("Lanczos: eigenvalue           = %e\n", esolver->evalue[i]);
 #endif
-#ifdef _LONG_DOUBLE
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: relative residual 2-norm = %Le\n",esolver2->resid);
+#endif	  
+#ifdef _LONG__LONG
+	  if( output ) printf("Lanczos: number of iterations = %lld\n",esolver2->iter[0]);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: relative residual 2-norm = %e\n",esolver2->resid);
+	  if( output ) printf("Lanczos: number of iterations = %d\n",esolver2->iter[0]);
 #endif
+#ifdef _COMPLEX	  
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: relative residual    = %Le\n\n",creall(esolver2->resid[0]));
+#else
+	  if( output ) printf("Lanczos: relative residual    = %e\n\n",creal(esolver2->resid[0]));
+#endif
+#else
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: relative residual    = %Le\n\n",esolver2->resid[0]);
+#else
+	  if( output ) printf("Lanczos: relative residual    = %e\n\n",esolver2->resid[0]);
+#endif
+#endif	  
 	}
     }
   esolver->evalue[0] = evalue0; 
-  esolver->iter = iter0;
-  esolver->resid = resid0;
-  esolver->residual = residual0;
-  lis_vector_copy(esolver->evector[esolver->options[LIS_EOPTIONS_MODE]], esolver->x);
+  esolver->iter[0] = iter0;
+  esolver->resid[0] = resid0;
 
-  lis_iesolver_destroy(esolver2); 
+  lis_vector_copy(esolver->evector[0], esolver->x);
+
+  lis_esolver_destroy(esolver2); 
 
   lis_free(t); 
-  lis_free(tx);
   lis_free(tq);
   lis_free(tr);
 
@@ -354,21 +376,19 @@ LIS_INT lis_eli(LIS_ESOLVER esolver)
 #define __FUNC__ "lis_eli_quad"
 LIS_INT lis_eli_quad(LIS_ESOLVER esolver)
 {
-  LIS_MATRIX        A;
-  LIS_VECTOR        x;
-  LIS_SCALAR        lshift;
-  LIS_INT               ss;
-  LIS_INT               emaxiter,iter0;
-  LIS_REAL          tol;
-  LIS_INT               i,j,k;
-  LIS_INT               output, niesolver;
-  LIS_REAL          nrm2,dot,resid,resid0,*residual0;
-  LIS_VECTOR        *v,r;
-  LIS_SCALAR        *t, *tx, tevalue, *tq, *tr, evalue, evalue0;
-  LIS_SOLVER        solver;
-  LIS_ESOLVER       esolver2;
-  char              esolvername[128], solvername[128], preconname[128];
-  LIS_INT           nsol, precon_type;
+  LIS_MATRIX A;
+  LIS_INT ss,ic;
+  LIS_INT emaxiter,iter0,qriter;
+  LIS_REAL tol,qrerr;
+  LIS_INT i,j,k;
+  LIS_INT output, niesolver;
+  LIS_REAL nrm2,dot,resid0;
+  LIS_VECTOR *v,r;
+  LIS_SCALAR *t,*tq,*tr,evalue,evalue0;
+  LIS_SOLVER solver;
+  LIS_ESOLVER esolver2;
+  char esolvername[128],solvername[128],preconname[128];
+  LIS_INT nsol,precon_type;
 
   ss = esolver->options[LIS_EOPTIONS_SUBSPACE];
   emaxiter = esolver->options[LIS_EOPTIONS_MAXITER];
@@ -377,35 +397,27 @@ LIS_INT lis_eli_quad(LIS_ESOLVER esolver)
   niesolver = esolver->options[LIS_EOPTIONS_INNER_ESOLVER];
 
   t = (LIS_SCALAR *)lis_malloc(ss*ss*sizeof(LIS_SCALAR), "lis_eli::t");
-  tx = (LIS_SCALAR *)lis_malloc(ss*sizeof(LIS_SCALAR), "lis_eli::tx");
   tq = (LIS_SCALAR *)lis_malloc(ss*ss*sizeof(LIS_SCALAR), "lis_eli::tq");
   tr = (LIS_SCALAR *)lis_malloc(ss*ss*sizeof(LIS_SCALAR), "lis_eli::tr");
   
   A = esolver->A;
-  x = esolver->work[0];
-  r = esolver->work[1];
-  v = &esolver->work[2];
+  r = esolver->work[0];
+  v = &esolver->work[1];
   lis_vector_set_all(0.0,v[0]);
   lis_vector_set_all(1.0,r);
   lis_vector_nrm2(r, &nrm2);
   
   lis_solver_create(&solver);
-  lis_solver_set_option("-i cg -p ilu",solver);
+  lis_solver_set_option("-i bicg -p none",solver);
   lis_solver_set_optionC(solver);
   lis_solver_get_solver(solver, &nsol);
   lis_solver_get_precon(solver, &precon_type);
   lis_solver_get_solvername(nsol, solvername);
   lis_solver_get_preconname(precon_type, preconname);
   lis_esolver_get_esolvername(niesolver, esolvername);
-#ifdef _LONGLONG
-  if( A->my_rank==0 ) printf("iesolver   : %s %lld\n", esolvername, niesolver);
-  if( A->my_rank==0 ) printf("solver     : %s %lld\n", solvername, nsol);
-  if( A->my_rank==0 ) printf("precon     : %s %lld\n", preconname, precon_type);
-#else
-  if( A->my_rank==0 ) printf("iesolver   : %s %d\n", esolvername, niesolver);
-  if( A->my_rank==0 ) printf("solver     : %s %d\n", solvername, nsol);
-  if( A->my_rank==0 ) printf("precon     : %s %d\n", preconname, precon_type);
-#endif
+  if( A->my_rank==0 ) printf("inner eigensolver     : %s\n", esolvername);
+  if( A->my_rank==0 ) printf("linear solver         : %s\n", solvername);
+  if( A->my_rank==0 ) printf("preconditioner        : %s\n", preconname);
 
   for (i=0;i<ss*ss;i++) t[i] = 0.0;
 
@@ -413,11 +425,11 @@ LIS_INT lis_eli_quad(LIS_ESOLVER esolver)
   while (j<ss-1)
     {
       j = j+1;
-
-      /* v(j) = r / beta(j-1) */
       lis_vector_copy(r, v[j]);
 
+      /* v(j) = r / beta(j-1) */
       /* r = A * v(j) */
+      /* r = r - beta(j-1) * v(j-1) */
       if (j==1) {
 	lis_vector_scale(1/nrm2, v[j]);
 	lis_matvec(A, v[j], r);
@@ -428,10 +440,10 @@ LIS_INT lis_eli_quad(LIS_ESOLVER esolver)
 	lis_vector_axpy(-t[(j-2)*ss+j-1], v[j-1], r); 
       }
 
-      /* alpha(j)  = v(j) * r */
+      /* alpha(j)  = <v(j), r> */
       lis_vector_dot(v[j], r, &t[(j-1)*ss+j-1]); 
 
-      /* r = r - v(j) * alpha(j) */
+      /* r = r - alpha(j) * v(j) */
       lis_vector_axpy(-t[(j-1)*ss+j-1], v[j], r); 
 
       /* reorthogonalization */
@@ -445,107 +457,138 @@ LIS_INT lis_eli_quad(LIS_ESOLVER esolver)
       lis_vector_nrm2(r, &t[(j-1)*ss+j]);
 
       /* convergence check */
-      if (t[(j-1)*ss+j]<tol) break;  
+      if (fabs(t[(j-1)*ss+j])<tol) break;  
       t[j*ss+j-1] = t[(j-1)*ss+j];
     }
 
-  /* compute approximate eigenvalues of a real symmetric
-     tridiagonal matrix T(j) = ST'(j)S^* */
-  lis_array_qr(ss, t, tq, tr); 
+  /* compute eigenvalues of a real symmetric tridiagonal matrix 
+     T(j) = ST'(j)S^* */
+  lis_array_qr(ss,t,tq,tr,&qriter,&qrerr); 
 
   for (i=0;i<ss;i++)
     {
       esolver->evalue[i] = t[i*ss+i];
     }
-  lis_sort_d(0, ss-1, esolver->evalue);
 
   if( A->my_rank==0 ) 
     {
-#ifdef _LONGLONG
-      printf("size of subspace = %lld\n", ss);
+#ifdef _LONG__LONG
+      printf("size of subspace = %lld\n\n", ss);
 #else
-      printf("size of subspace = %d\n", ss);
+      printf("size of subspace = %d\n\n", ss);
 #endif
-      if( output & LIS_PRINT_OUT ) printf("solutions in subspace:\n");
+      if( output & LIS_PRINT_OUT ) printf("approximate eigenvalues in subspace:\n\n");
       for (i=0;i<ss;i++)
 	{
-#ifdef _LONGLONG
-	  if( output & LIS_PRINT_OUT ) if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %lld\n", i);
+#ifdef _LONG__LONG
+	  if( output ) if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %lld\n", i);
 #else
-	  if( output & LIS_PRINT_OUT ) if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %d\n", i);
+	  if( output ) if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %d\n", i);
 #endif
-#ifdef _LONG_DOUBLE
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %Le\n", esolver->evalue[i]);
+#ifdef _COMPLEX
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue               = %Le + %Le * I\n", creall(esolver->evalue[i]), cimagl(esolver->evalue[i]));
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %e\n", esolver->evalue[i]);
+	  if( output ) printf("Lanczos: eigenvalue               = %e + %e * I\n", creal(esolver->evalue[i]), cimag(esolver->evalue[i]));
 #endif
+#else	  
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue               = %Le\n", esolver->evalue[i]);
+#else
+	  if( output ) printf("Lanczos: eigenvalue               = %e\n", esolver->evalue[i]);
+#endif
+#endif	  
 	}
-      if( output & LIS_PRINT_OUT ) printf("refined solutions:\n");
+      if( output ) printf("\n");
+      if( output ) printf("compute refined eigenpairs:\n\n");
     }
 
   lis_esolver_create(&esolver2);
   esolver2->options[LIS_EOPTIONS_ESOLVER] = niesolver;
   esolver2->options[LIS_EOPTIONS_SUBSPACE] = 1;
   esolver2->options[LIS_EOPTIONS_MAXITER] = emaxiter;
+  esolver2->options[LIS_EOPTIONS_OUTPUT] = esolver->options[LIS_EOPTIONS_OUTPUT];
   esolver2->params[LIS_EPARAMS_RESID - LIS_EOPTIONS_LEN] = tol; 
   esolver2->eprecision = LIS_PRECISION_QUAD;
 
-  /* compute approximate eigenvectors X = V(j)S */
+  /* compute refined eigenpairs */
   for (i=0;i<ss;i++)
     {
       lis_vector_duplicate(A, &esolver->evector[i]); 
       esolver2->lshift = -(esolver->evalue[i]);
       lis_esolve(A, esolver->evector[i], &evalue, esolver2);
       esolver->evalue[i] = evalue - esolver2->lshift;
+      esolver->iter[i] = esolver2->iter[0];      
+      esolver->resid[i] = esolver2->resid[0];
 
       if (i==0) 
 	{
 	  evalue0 = esolver->evalue[0];
-	  iter0 = esolver2->iter;
-	  resid0 = esolver2->resid;
-	  residual0 = esolver2->residual;
-	  esolver->ptimes = esolver2->ptimes;
-	  esolver->itimes = esolver2->itimes;
-	  esolver->p_c_times = esolver2->p_c_times;
-	  esolver->p_i_times = esolver2->p_i_times;
+	  iter0 = esolver2->iter[0];
+	  resid0 = esolver2->resid[0];
+	  if( output & LIS_EPRINT_MEM ) 
+	    {
+	      for (ic=0;ic<iter0+1;ic++)
+		{
+		  esolver->rhistory[ic] = esolver2->rhistory[ic]; 
+		}
+	    }
+	  esolver->ptime = esolver2->ptime;
+	  esolver->itime = esolver2->itime;
+	  esolver->p_c_time = esolver2->p_c_time;
+	  esolver->p_i_time = esolver2->p_i_time;
 	}
 
       if (A->my_rank==0) 
 	{
 
-#ifdef _LONGLONG
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %lld\n", i);
+#ifdef _LONG__LONG
+	  if( output ) printf("Lanczos: mode number          = %lld\n", i);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: mode number              = %d\n", i);
+	  if( output ) printf("Lanczos: mode number          = %d\n", i);
 #endif
-#ifdef _LONG_DOUBLE
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %Le\n", esolver->evalue[i]);
+#ifdef _COMPLEX	  
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue           = %Le + %Le * I\n", creall(esolver->evalue[i]), cimagl(esolver->evalue[i]));
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: eigenvalue               = %e\n", esolver->evalue[i]);
+	  if( output ) printf("Lanczos: eigenvalue           = %e + %e * I\n", creal(esolver->evalue[i]), cimag(esolver->evalue[i]));
 #endif
-#ifdef _LONGLONG
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: number of iterations     = %lld\n",esolver2->iter);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: number of iterations     = %d\n",esolver2->iter);
-#endif
-#ifdef _LONG_DOUBLE
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: relative residual 2-norm = %Le\n",esolver2->resid);
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: eigenvalue           = %Le\n", esolver->evalue[i]);
 #else
-	  if( output & LIS_PRINT_OUT ) printf("Lanczos: relative residual 2-norm = %e\n",esolver2->resid);
+	  if( output ) printf("Lanczos: eigenvalue           = %e\n", esolver->evalue[i]);
 #endif
+#endif
+#ifdef _LONG__LONG
+	  if( output ) printf("Lanczos: number of iterations = %lld\n",esolver2->iter[0]);
+#else
+	  if( output ) printf("Lanczos: number of iterations = %d\n",esolver2->iter[0]);
+#endif
+#ifdef _COMPLEX	  
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: relative residual    = %Le\n\n",creall(esolver2->resid[0]));
+#else
+	  if( output ) printf("Lanczos: relative residual    = %e\n\n",creal(esolver2->resid[0]));
+#endif
+#else
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("Lanczos: relative residual    = %Le\n\n",esolver2->resid[0]);
+#else
+	  if( output ) printf("Lanczos: relative residual    = %e\n\n",esolver2->resid[0]);
+#endif
+#endif	  
 	}
     }
   esolver->evalue[0] = evalue0; 
-  esolver->iter = iter0;
-  esolver->resid = resid0;
-  esolver->residual = residual0;
+  esolver->iter[0] = iter0;
+  esolver->resid[0] = resid0;
 
-  lis_vector_copy(esolver->evector[esolver->options[LIS_EOPTIONS_MODE]], esolver->x);
+  lis_vector_copy(esolver->evector[0], esolver->x);
 
   lis_esolver_destroy(esolver2); 
 
   lis_free(t); 
-  lis_free(tx);
   lis_free(tq);
   lis_free(tr);
 
