@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -64,8 +64,9 @@
  * lis_matrix_convert          | o   |     |
  * lis_matrix_copy             | o   | o   |
  * lis_matrix_get_diagonal     | o   | o   |
- * lis_matrix_scaling          | o   | o   |
- * lis_matrix_scaling_symm     | o   | o   |
+ * lis_matrix_shift_diagonal   | o   | o   |
+ * lis_matrix_scale            | o   | o   |
+ * lis_matrix_scale_symm       | o   | o   |
  * lis_matrix_normf            | o   | o   |
  * lis_matrix_sort             | o   | o   |
  * lis_matrix_solve            | xxx | o   |
@@ -91,10 +92,9 @@ LIS_INT lis_matrix_set_csc(LIS_INT nnz, LIS_INT *ptr, LIS_INT *index, LIS_SCALAR
 
 #undef __FUNC__
 #define __FUNC__ "lis_matrix_setDLU_csc"
-LIS_INT lis_matrix_setDLU_csc(LIS_INT nnzl, LIS_INT nnzu, LIS_SCALAR *diag, LIS_INT *lptr, LIS_INT *lindex, LIS_SCALAR *lvalue,
-						  LIS_INT *uptr, LIS_INT *uindex, LIS_SCALAR *uvalue, LIS_MATRIX A)
+LIS_INT lis_matrix_setDLU_csc(LIS_INT nnzl, LIS_INT nnzu, LIS_SCALAR *diag, LIS_INT *lptr, LIS_INT *lindex, LIS_SCALAR *lvalue, LIS_INT *uptr, LIS_INT *uindex, LIS_SCALAR *uvalue, LIS_MATRIX A)
 {
-	LIS_INT				err;
+	LIS_INT err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -112,6 +112,7 @@ LIS_INT lis_matrix_setDLU_csc(LIS_INT nnzl, LIS_INT nnzu, LIS_SCALAR *diag, LIS_
 LIS_INT lis_matrix_malloc_csc(LIS_INT n, LIS_INT nnz, LIS_INT **ptr, LIS_INT **index, LIS_SCALAR **value)
 {
 	LIS_INT err;
+	
 	LIS_DEBUG_FUNC_IN;
 
 	err = lis_matrix_malloc_csr(n,nnz,ptr,index,value);
@@ -123,10 +124,9 @@ LIS_INT lis_matrix_malloc_csc(LIS_INT n, LIS_INT nnz, LIS_INT **ptr, LIS_INT **i
 
 #undef __FUNC__
 #define __FUNC__ "lis_matrix_elements_copy_csc"
-LIS_INT lis_matrix_elements_copy_csc(LIS_INT np, LIS_INT *ptr, LIS_INT *index, LIS_SCALAR *value,
-								 LIS_INT *o_ptr, LIS_INT *o_index, LIS_SCALAR *o_value)
+LIS_INT lis_matrix_elements_copy_csc(LIS_INT np, LIS_INT *ptr, LIS_INT *index, LIS_SCALAR *value, LIS_INT *o_ptr, LIS_INT *o_index, LIS_SCALAR *o_value)
 {
-	LIS_INT			i,j;
+	LIS_INT	i,j;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -162,12 +162,12 @@ LIS_INT lis_matrix_elements_copy_csc(LIS_INT np, LIS_INT *ptr, LIS_INT *index, L
 #define __FUNC__ "lis_matrix_copy_csc"
 LIS_INT lis_matrix_copy_csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			err;
-	LIS_INT			i,n,np,nnz,lnnz,unnz;
-	LIS_INT			*ptr,*index;
-	LIS_INT			*lptr,*lindex;
-	LIS_INT			*uptr,*uindex;
-	LIS_SCALAR	*value,*lvalue,*uvalue,*diag;
+	LIS_INT err;
+	LIS_INT i,n,np,nnz,lnnz,unnz;
+	LIS_INT *ptr,*index;
+	LIS_INT *lptr,*lindex;
+	LIS_INT *uptr,*uindex;
+	LIS_SCALAR *value,*lvalue,*uvalue,*diag;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -295,8 +295,8 @@ LIS_INT lis_matrix_get_diagonal_csc(LIS_MATRIX A, LIS_SCALAR d[])
 }
 
 #undef __FUNC__
-#define __FUNC__ "lis_matrix_scaling_csc"
-LIS_INT lis_matrix_scaling_csc(LIS_MATRIX A, LIS_SCALAR d[])
+#define __FUNC__ "lis_matrix_shift_diagonal_csc"
+LIS_INT lis_matrix_shift_diagonal_csc(LIS_MATRIX A, LIS_SCALAR alpha)
 {
 	LIS_INT i,j;
 	LIS_INT n,np;
@@ -304,6 +304,47 @@ LIS_INT lis_matrix_scaling_csc(LIS_MATRIX A, LIS_SCALAR d[])
 	LIS_DEBUG_FUNC_IN;
 
 	n    = A->n;
+	np   = A->np;
+	if( A->is_splited )
+	{
+		#ifdef _OPENMP
+		#pragma omp parallel for private(i,j)
+		#endif
+		for(i=0; i<n; i++)
+		{
+			A->D->value[i] += alpha;
+		}
+	}
+	else
+	{
+		#ifdef _OPENMP
+		#pragma omp parallel for private(i,j)
+		#endif
+		for(i=0; i<np; i++)
+		{
+			for(j=A->ptr[i];j<A->ptr[i+1];j++)
+			{
+				if( i==A->index[j] )
+				{
+					A->value[j] += alpha;
+					break;
+				}
+			}
+		}
+	}
+	LIS_DEBUG_FUNC_OUT;
+	return LIS_SUCCESS;
+}
+
+#undef __FUNC__
+#define __FUNC__ "lis_matrix_scale_csc"
+LIS_INT lis_matrix_scale_csc(LIS_MATRIX A, LIS_SCALAR d[])
+{
+	LIS_INT i,j;
+	LIS_INT np;
+
+	LIS_DEBUG_FUNC_IN;
+
 	np   = A->np;
 	if( A->is_splited )
 	{
@@ -341,15 +382,14 @@ LIS_INT lis_matrix_scaling_csc(LIS_MATRIX A, LIS_SCALAR d[])
 }
 
 #undef __FUNC__
-#define __FUNC__ "lis_matrix_scaling_symm_csc"
-LIS_INT lis_matrix_scaling_symm_csc(LIS_MATRIX A, LIS_SCALAR d[])
+#define __FUNC__ "lis_matrix_scale_symm_csc"
+LIS_INT lis_matrix_scale_symm_csc(LIS_MATRIX A, LIS_SCALAR d[])
 {
 	LIS_INT i,j;
-	LIS_INT n,np;
+	LIS_INT np;
 
 	LIS_DEBUG_FUNC_IN;
 
-	n    = A->n;
 	np   = A->np;
 	if( A->is_splited )
 	{
@@ -391,12 +431,11 @@ LIS_INT lis_matrix_scaling_symm_csc(LIS_MATRIX A, LIS_SCALAR d[])
 LIS_INT lis_matrix_normf_csc(LIS_MATRIX A, LIS_SCALAR *nrm)
 {
 	LIS_INT i,j;
-	LIS_INT n,np;
+	LIS_INT np;
 	LIS_SCALAR sum;
 
 	LIS_DEBUG_FUNC_IN;
 
-	n    = A->n;
 	np   = A->np;
 	sum  = (LIS_SCALAR)0;
 	if( A->is_splited )
@@ -439,7 +478,7 @@ LIS_INT lis_matrix_normf_csc(LIS_MATRIX A, LIS_SCALAR *nrm)
 #define __FUNC__ "lis_matrix_transpose_csc"
 LIS_INT lis_matrix_transpose_csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			err;
+	LIS_INT err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -453,20 +492,19 @@ LIS_INT lis_matrix_transpose_csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 #define __FUNC__ "lis_matrix_split_csc"
 LIS_INT lis_matrix_split_csc(LIS_MATRIX A)
 {
-	LIS_INT				i,j,n,np;
-	LIS_INT				nnzl,nnzu;
-	LIS_INT				err;
-	LIS_INT				*lptr,*lindex,*uptr,*uindex;
-	LIS_SCALAR		*lvalue,*uvalue;
+	LIS_INT i,j,np;
+	LIS_INT nnzl,nnzu;
+	LIS_INT err;
+	LIS_INT *lptr,*lindex,*uptr,*uindex;
+	LIS_SCALAR *lvalue,*uvalue;
 	LIS_MATRIX_DIAG	D;
 	#ifdef _OPENMP
-		LIS_INT				kl,ku;
-		LIS_INT				*liw,*uiw;
+		LIS_INT kl,ku;
+		LIS_INT *liw,*uiw;
 	#endif
 
 	LIS_DEBUG_FUNC_IN;
 
-	n        = A->n;
 	np       = A->np;
 	nnzl     = 0;
 	nnzu     = 0;
@@ -643,11 +681,11 @@ LIS_INT lis_matrix_split_csc(LIS_MATRIX A)
 #define __FUNC__ "lis_matrix_merge_csc"
 LIS_INT lis_matrix_merge_csc(LIS_MATRIX A)
 {
-	LIS_INT				i,j,n,np;
-	LIS_INT				nnz;
-	LIS_INT				err;
-	LIS_INT				*ptr,*index;
-	LIS_SCALAR		*value;
+	LIS_INT i,j,n,np;
+	LIS_INT nnz;
+	LIS_INT err;
+	LIS_INT *ptr,*index;
+	LIS_SCALAR *value;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -700,13 +738,12 @@ LIS_INT lis_matrix_merge_csc(LIS_MATRIX A)
 #define __FUNC__ "lis_matrix_sort_csc"
 LIS_INT lis_matrix_sort_csc(LIS_MATRIX A)
 {
-	LIS_INT i,n,np;
+	LIS_INT i,np;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( !A->is_sorted )
 	{
-		n  = A->n;
 		np = A->np;
 		if( A->is_splited )
 		{
@@ -741,14 +778,13 @@ LIS_INT lis_matrix_sort_csc(LIS_MATRIX A)
 LIS_INT lis_matrix_solve_csc(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT flag)
 {
 	LIS_INT i,j,n,np;
-	LIS_SCALAR	t;
-	LIS_SCALAR	*b,*x;
+	LIS_SCALAR t;
+	LIS_SCALAR *x;
 
 	LIS_DEBUG_FUNC_IN;
 
 	n  = A->n;
 	np = A->np;
-	b  = B->value;
 	x  = X->value;
 
 	lis_vector_copy(B,X);
@@ -803,13 +839,12 @@ LIS_INT lis_matrix_solve_csc(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT f
 #define __FUNC__ "lis_matrix_solvet_csc"
 LIS_INT lis_matrix_solvet_csc(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT flag)
 {
-	LIS_INT i,j,n,np;
-	LIS_SCALAR	t;
-	LIS_SCALAR	*b,*x;
+	LIS_INT i,j,np;
+	LIS_SCALAR t;
+	LIS_SCALAR *b,*x;
 
 	LIS_DEBUG_FUNC_IN;
 
-	n  = A->n;
 	np = A->np;
 	b  = B->value;
 	x  = X->value;
@@ -868,15 +903,14 @@ LIS_INT lis_matrix_solvet_csc(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT
 #define __FUNC__ "lis_matrix_convert_csr2csc"
 LIS_INT lis_matrix_convert_csr2csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			i,j,jj,k,l;
-	LIS_INT			err;
-	LIS_INT			n,nnz,np;
-	LIS_INT			is,ie;
-	LIS_INT			*iw,*iw2;
-	LIS_INT			*ptr,*index;
-	LIS_SCALAR	*value;
+	LIS_INT i,j,jj,k,l;
+	LIS_INT err;
+	LIS_INT n,nnz,np;
+	LIS_INT *iw,*iw2;
+	LIS_INT *ptr,*index;
+	LIS_SCALAR *value;
 	#ifdef _OPENMP
-		LIS_INT			my_rank,nprocs;
+		LIS_INT my_rank,nprocs;
 	#endif
 
 	LIS_DEBUG_FUNC_IN;
@@ -884,8 +918,6 @@ LIS_INT lis_matrix_convert_csr2csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 	n       = Ain->n;
 	np      = Ain->np;
 	nnz     = Ain->nnz;
-	is      = Ain->is;
-	ie      = Ain->ie;
 	#ifdef _OPENMP
 		nprocs  = omp_get_max_threads();
 	#endif
@@ -935,7 +967,7 @@ LIS_INT lis_matrix_convert_csr2csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 
 	/* convert csc */
 	#ifdef _OPENMP
-		#pragma omp parallel private(i,j,jj,k,is,ie,l,my_rank)
+		#pragma omp parallel private(i,j,jj,k,l,my_rank)
 		{
 			my_rank  = omp_get_thread_num();
 			#pragma omp for
@@ -1058,14 +1090,14 @@ LIS_INT lis_matrix_convert_csr2csc(LIS_MATRIX Ain, LIS_MATRIX Aout)
 #define __FUNC__ "lis_matrix_convert_csc2csr"
 LIS_INT lis_matrix_convert_csc2csr(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			i,j,jj,k,l;
-	LIS_INT			err;
-	LIS_INT			n,np,nnz;
-	LIS_INT			*iw,*iw2;
-	LIS_INT			*ptr,*index;
-	LIS_SCALAR	*value;
+	LIS_INT i,j,jj,k,l;
+	LIS_INT err;
+	LIS_INT n,np,nnz;
+	LIS_INT *iw,*iw2;
+	LIS_INT *ptr,*index;
+	LIS_SCALAR *value;
 	#ifdef _OPENMP
-		LIS_INT			nprocs,my_rank;
+		LIS_INT nprocs,my_rank;
 	#endif
 
 	LIS_DEBUG_FUNC_IN;
