@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -63,9 +63,10 @@
  *-----------------------------+-----+-----|
  * lis_matrix_convert          | o   |     |
  * lis_matrix_copy             | o   | o   |
- * lis_matrix_get_jadgonal     | o   | o   |
- * lis_matrix_scaling          | o   | o   |
- * lis_matrix_scaling_symm     | o   | o   |
+ * lis_matrix_get_diagonal     | o   | o   |
+ * lis_matrix_shift_diagonal   | o   | o   |
+ * lis_matrix_scale            | o   | o   |
+ * lis_matrix_scale_symm       | o   | o   |
  * lis_matrix_normf            | o   | o   |
  * lis_matrix_sort             | o   | o   |
  * lis_matrix_solve            | xxx | o   |
@@ -122,11 +123,10 @@ LIS_INT lis_matrix_set_jad(LIS_INT nnz, LIS_INT maxnzr, LIS_INT *perm, LIS_INT *
 
 #undef __FUNC__
 #define __FUNC__ "lis_matrix_setDLU_jad"
-LIS_INT lis_matrix_setDLU_jad(LIS_INT lnnz, LIS_INT unnz, LIS_INT lmaxnzr, LIS_INT umaxnzr, LIS_SCALAR *diag, LIS_INT *lperm, LIS_INT *lptr, LIS_INT *lindex, LIS_SCALAR *lvalue,
-						  LIS_INT *uperm, LIS_INT *uptr, LIS_INT *uindex, LIS_SCALAR *uvalue, LIS_MATRIX A)
+LIS_INT lis_matrix_setDLU_jad(LIS_INT lnnz, LIS_INT unnz, LIS_INT lmaxnzr, LIS_INT umaxnzr, LIS_SCALAR *diag, LIS_INT *lperm, LIS_INT *lptr, LIS_INT *lindex, LIS_SCALAR *lvalue, LIS_INT *uperm, LIS_INT *uptr, LIS_INT *uindex, LIS_SCALAR *uvalue, LIS_MATRIX A)
 {
-	LIS_INT				n,i,err;
-	LIS_INT				*lcol,*ucol;
+	LIS_INT n,i,err;
+	LIS_INT *lcol,*ucol;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -263,11 +263,10 @@ LIS_INT lis_matrix_malloc_jad(LIS_INT n, LIS_INT nnz, LIS_INT maxnzr, LIS_INT **
 
 #undef __FUNC__
 #define __FUNC__ "lis_matrix_elements_copy_jad"
-LIS_INT lis_matrix_elements_copy_jad(LIS_INT n, LIS_INT maxnzr, LIS_INT *perm, LIS_INT *ptr, LIS_INT *index, LIS_SCALAR *value,
-								 LIS_INT *o_perm, LIS_INT *o_ptr, LIS_INT *o_index, LIS_SCALAR *o_value)
+LIS_INT lis_matrix_elements_copy_jad(LIS_INT n, LIS_INT maxnzr, LIS_INT *perm, LIS_INT *ptr, LIS_INT *index, LIS_SCALAR *value, LIS_INT *o_perm, LIS_INT *o_ptr, LIS_INT *o_index, LIS_SCALAR *o_value)
 {
-	LIS_INT			i,j,is,ie;
-	LIS_INT			nprocs,my_rank;
+	LIS_INT i,j,is,ie;
+	LIS_INT nprocs,my_rank;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -314,12 +313,12 @@ LIS_INT lis_matrix_elements_copy_jad(LIS_INT n, LIS_INT maxnzr, LIS_INT *perm, L
 #define __FUNC__ "lis_matrix_copy_jad"
 LIS_INT lis_matrix_copy_jad(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			err;
-	LIS_INT			i,n,nnz,lnnz,unnz,maxnzr,lmaxnzr,umaxnzr;
-	LIS_INT			*perm,*ptr,*index;
-	LIS_INT			*lperm,*lptr,*lindex;
-	LIS_INT			*uperm,*uptr,*uindex;
-	LIS_SCALAR	*value,*lvalue,*uvalue,*diag;
+	LIS_INT err;
+	LIS_INT i,n,nnz,lnnz,unnz,maxnzr,lmaxnzr,umaxnzr;
+	LIS_INT *perm,*ptr,*index;
+	LIS_INT *lperm,*lptr,*lindex;
+	LIS_INT *uperm,*uptr,*uindex;
+	LIS_SCALAR *value,*lvalue,*uvalue,*diag;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -484,8 +483,83 @@ LIS_INT lis_matrix_get_diagonal_jad(LIS_MATRIX A, LIS_SCALAR d[])
 }
 
 #undef __FUNC__
-#define __FUNC__ "lis_matrix_scaling_jad"
-LIS_INT lis_matrix_scaling_jad(LIS_MATRIX A, LIS_SCALAR d[])
+#define __FUNC__ "lis_matrix_shift_diagonal_jad"
+LIS_INT lis_matrix_shift_diagonal_jad(LIS_MATRIX A, LIS_SCALAR alpha)
+{
+	LIS_INT i,j,k,l;
+	LIS_INT n,maxnzr;
+	#ifdef _OPENMP
+		LIS_INT is,ie,js,je;
+		LIS_INT nprocs,my_rank;
+	#endif
+
+	LIS_DEBUG_FUNC_IN;
+
+	n      = A->n;
+	maxnzr = A->maxnzr;
+	k      = n;
+	if( A->is_splited )
+	{
+		#ifdef _OPENMP
+		#pragma omp parallel for private(i)
+		#endif
+		for(i=0; i<n; i++)
+		{
+			A->D->value[i] += alpha;
+		}
+	}
+	else
+	{
+		#ifdef _OPENMP
+			nprocs = omp_get_max_threads();
+			#pragma omp parallel private(i,j,k,l,is,ie,js,je,my_rank)
+			{
+				my_rank = omp_get_thread_num();
+				LIS_GET_ISIE(my_rank,nprocs,n,is,ie);
+				k = ie-is;
+				for(j=0;j<maxnzr;j++)
+				{
+					l  = is;
+					js = A->ptr[my_rank*(maxnzr+1) + j];
+					je = A->ptr[my_rank*(maxnzr+1) + j+1];
+					for(i=js;i<je;i++)
+					{
+						if( A->row[l]==A->index[i] )
+						{
+							A->value[i] += alpha;
+							k--;
+							if( k==0 ) goto get_diag_end;
+						}
+						l++;
+					}
+				}
+				get_diag_end:
+				;
+			}
+		#else
+			for(j=0;j<maxnzr;j++)
+			{
+				l = 0;
+				for(i=A->ptr[j];i<A->ptr[j+1];i++)
+				{
+					if( A->row[l]==A->index[i] )
+					{
+						A->value[i] += alpha;
+						k--;
+						if( k==0 ) return LIS_SUCCESS;
+					}
+					l++;
+				}
+			}
+		#endif
+	}
+	LIS_DEBUG_FUNC_OUT;
+	return LIS_SUCCESS;
+}
+
+#undef __FUNC__
+#define __FUNC__ "lis_matrix_scale_jad"
+LIS_INT lis_matrix_scale_jad(LIS_MATRIX A, LIS_SCALAR d[])
 {
 	LIS_INT i,j,k,is,ie,js,je;
 	LIS_INT n,maxnzr;
@@ -577,8 +651,8 @@ LIS_INT lis_matrix_scaling_jad(LIS_MATRIX A, LIS_SCALAR d[])
 }
 
 #undef __FUNC__
-#define __FUNC__ "lis_matrix_scaling_symm_jad"
-LIS_INT lis_matrix_scaling_symm_jad(LIS_MATRIX A, LIS_SCALAR d[])
+#define __FUNC__ "lis_matrix_scale_symm_jad"
+LIS_INT lis_matrix_scale_symm_jad(LIS_MATRIX A, LIS_SCALAR d[])
 {
 	LIS_INT i,j,k,is,ie,js,je;
 	LIS_INT n,maxnzr;
@@ -737,23 +811,22 @@ LIS_INT lis_matrix_transpose_jad(LIS_MATRIX Ain, LIS_MATRIX *Aout)
 #define __FUNC__ "lis_matrix_split_jad"
 LIS_INT lis_matrix_split_jad(LIS_MATRIX A)
 {
-	LIS_INT				i,j,k,kk,n,nnz,maxnzr;
-	LIS_INT				lnnz,unnz,lmaxnzr,umaxnzr;
-	LIS_INT				err;
-	LIS_INT				*liw,*uiw,*liw2,*uiw2;
-	LIS_INT				*lperm,*lptr,*lindex,*lcol;
-	LIS_INT				*uperm,*uptr,*uindex,*ucol;
+	LIS_INT i,j,k,kk,n,maxnzr;
+	LIS_INT lnnz,unnz,lmaxnzr,umaxnzr;
+	LIS_INT err;
+	LIS_INT *liw,*uiw,*liw2,*uiw2;
+	LIS_INT *lperm,*lptr,*lindex;
+	LIS_INT *uperm,*uptr,*uindex;
 	#ifdef _OPENMP
-		LIS_INT				*iw;
-		LIS_INT				my_rank,nprocs,is,ie,js,je;
+		LIS_INT *iw;
+		LIS_INT my_rank,nprocs,is,ie,js,je;
 	#endif
-	LIS_SCALAR		*lvalue,*uvalue;
+	LIS_SCALAR *lvalue,*uvalue;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
 
 	n        = A->n;
-	nnz      = A->nnz;
 	maxnzr   = A->maxnzr;
 	lmaxnzr  = 0;
 	umaxnzr  = 0;
@@ -765,12 +838,10 @@ LIS_INT lis_matrix_split_jad(LIS_MATRIX A)
 	uiw2     = NULL;
 	D        = NULL;
 	lperm    = NULL;
-	lcol     = NULL;
 	lptr     = NULL;
 	lindex   = NULL;
 	lvalue   = NULL;
 	uperm    = NULL;
-	ucol     = NULL;
 	uptr     = NULL;
 	uindex   = NULL;
 	uvalue   = NULL;
@@ -1069,20 +1140,19 @@ LIS_INT lis_matrix_split_jad(LIS_MATRIX A)
 LIS_INT lis_matrix_merge_jad(LIS_MATRIX A)
 {
 	LIS_INT i,j,k,kk,ie;
-	LIS_INT n,nnz,maxnzr,nn;
-	LIS_INT				err;
-	LIS_INT				*perm,*ptr,*index,*iw,*iw2;
-	LIS_SCALAR		*value;
+	LIS_INT n,nnz,maxnzr;
+	LIS_INT err;
+	LIS_INT *perm,*ptr,*index,*iw,*iw2;
+	LIS_SCALAR *value;
 	#ifdef _OPENMP
 		LIS_INT is,js,je,nprocs,my_rank;
-		LIS_INT				*iw3;
+		LIS_INT *iw3;
 	#endif
 
 	LIS_DEBUG_FUNC_IN;
 
 
 	n       = A->n;
-	nn      = n;
 	perm    = NULL;
 	ptr     = NULL;
 	index   = NULL;
@@ -1360,8 +1430,8 @@ LIS_INT lis_matrix_sort_jad(LIS_MATRIX A)
 LIS_INT lis_matrix_solve_jad(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT flag)
 {
 	LIS_INT i,j,k,l,n;
-	LIS_SCALAR	t;
-	LIS_SCALAR	*b,*x;
+	LIS_SCALAR t;
+	LIS_SCALAR *b,*x;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -1440,13 +1510,12 @@ LIS_INT lis_matrix_solve_jad(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT f
 LIS_INT lis_matrix_solvet_jad(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT flag)
 {
 	LIS_INT i,j,k,l,n;
-	LIS_SCALAR	t;
-	LIS_SCALAR	*b,*x;
+	LIS_SCALAR t;
+	LIS_SCALAR *x;
 
 	LIS_DEBUG_FUNC_IN;
 
 	n  = A->n;
-	b  = B->value;
 	x  = X->value;
 
 	lis_vector_copy(B,X);
@@ -1518,19 +1587,18 @@ LIS_INT lis_matrix_solvet_jad(LIS_MATRIX A, LIS_VECTOR B, LIS_VECTOR X, LIS_INT
 #define __FUNC__ "lis_matrix_convert_csr2jad"
 LIS_INT lis_matrix_convert_csr2jad(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			i,j,l,js,je;
-	LIS_INT			err;
-	LIS_INT			gn,n,nnz,maxnzr,nprocs,my_rank;
-	LIS_INT			is,ie;
-	LIS_INT			*iw,*maxnzrpe,*nnzpe;
-	LIS_INT			*perm,*ptr,*index;
-	LIS_SCALAR	*value;
+	LIS_INT i,j,l,js,je;
+	LIS_INT err;
+	LIS_INT n,nnz,maxnzr,nprocs,my_rank;
+	LIS_INT is,ie;
+	LIS_INT *iw,*maxnzrpe,*nnzpe;
+	LIS_INT *perm,*ptr,*index;
+	LIS_SCALAR *value;
 
 	LIS_DEBUG_FUNC_IN;
 
 	n       = Ain->n;
-	gn      = Ain->gn;
-	nnz		= Ain->nnz;
+	nnz	= Ain->nnz;
 	my_rank = Ain->my_rank;
 	is      = Ain->is;
 	ie      = Ain->ie;
@@ -1700,16 +1768,16 @@ LIS_INT lis_matrix_convert_csr2jad(LIS_MATRIX Ain, LIS_MATRIX Aout)
 #define __FUNC__ "lis_matrix_convert_csr2jad"
 LIS_INT lis_matrix_convert_csr2jad(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			i,j,jj,k,kk,l,js,je;
-	LIS_INT			err;
-	LIS_INT			np,n,nnz,nnz2,maxnzr,maxnzr2,nprocs,my_rank;
-	LIS_INT			is,ie,pe;
-	LIS_INT			*iw,*maxnzrpe,*nnzpe;
-	LIS_INT			*iw2,*maxnzrpe2,*nnzpe2;
-	LIS_INT			*perm,*ptr,*index;
-	LIS_INT			*perm2,*ptr2,*index2;
-	LIS_SCALAR	*value;
-	LIS_SCALAR	*value2;
+	LIS_INT i,j,jj,k,kk,l,js,je;
+	LIS_INT err;
+	LIS_INT np,n,nnz,nnz2,maxnzr,maxnzr2,nprocs,my_rank;
+	LIS_INT is,ie,pe;
+	LIS_INT *iw,*maxnzrpe,*nnzpe;
+	LIS_INT *iw2,*maxnzrpe2,*nnzpe2;
+	LIS_INT *perm,*ptr,*index;
+	LIS_INT *perm2,*ptr2,*index2;
+	LIS_SCALAR *value;
+	LIS_SCALAR *value2;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -1973,14 +2041,14 @@ LIS_INT lis_matrix_convert_csr2jad(LIS_MATRIX Ain, LIS_MATRIX Aout)
 #define __FUNC__ "lis_matrix_convert_jad2csr"
 LIS_INT lis_matrix_convert_jad2csr(LIS_MATRIX Ain, LIS_MATRIX Aout)
 {
-	LIS_INT			i,j,jj,k,is,ie;
-	LIS_INT			err;
-	LIS_INT			n,nnz,maxnzr;
-	LIS_INT			*iw;
-	LIS_INT			*ptr,*index;
-	LIS_SCALAR	*value;
+	LIS_INT i,j,jj,k,is,ie;
+	LIS_INT err;
+	LIS_INT n,nnz,maxnzr;
+	LIS_INT *iw;
+	LIS_INT *ptr,*index;
+	LIS_SCALAR *value;
 	#ifdef _OPENMP
-		LIS_INT		nprocs,my_rank;
+		LIS_INT	nprocs,my_rank;
 	#endif
 
 	LIS_DEBUG_FUNC_IN;
@@ -2112,8 +2180,8 @@ LIS_INT lis_matrix_convert_jad2csr(LIS_MATRIX Ain, LIS_MATRIX Aout)
 #define __FUNC__ "lis_vector_sort_jad_order"
 LIS_INT lis_vector_sort_jad_order(LIS_MATRIX A, LIS_VECTOR v)
 {
-	LIS_INT			i,n,np;
-	LIS_SCALAR	*t;
+	LIS_INT i,n,np;
+	LIS_SCALAR *t;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -2147,8 +2215,8 @@ LIS_INT lis_vector_sort_jad_order(LIS_MATRIX A, LIS_VECTOR v)
 #define __FUNC__ "lis_vector_sort_ascending_order"
 LIS_INT lis_vector_sort_ascending_order(LIS_MATRIX A, LIS_VECTOR v)
 {
-	LIS_INT			i,n,np;
-	LIS_SCALAR	*t;
+	LIS_INT i,n,np;
+	LIS_SCALAR *t;
 
 	LIS_DEBUG_FUNC_IN;
 
