@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -52,6 +52,14 @@
 #endif
 #include "lislib.h"
 
+/************************************************
+ * lis_precon_create
+ * lis_precon_psd_create
+ * lis_precon_psd_update
+ * lis_psolve
+ * lis_psolvet
+ ************************************************/
+
 LIS_INT lis_symbolic_fact_csr(LIS_SOLVER solver, LIS_PRECON precon);
 LIS_INT lis_numerical_fact_csr(LIS_SOLVER solver, LIS_PRECON precon);
 LIS_INT lis_symbolic_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon);
@@ -63,13 +71,12 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon);
 #define __FUNC__ "lis_precon_create_iluk"
 LIS_INT lis_precon_create_iluk(LIS_SOLVER solver, LIS_PRECON precon)
 {
-	LIS_INT				storage,block,err;
-	LIS_MATRIX		A,B;
+	LIS_INT	storage,err;
+	LIS_MATRIX A,B;
 
 	LIS_DEBUG_FUNC_IN;
 
 	storage     = solver->options[LIS_OPTIONS_STORAGE];
-	block       = solver->options[LIS_OPTIONS_STORAGE_BLOCK];
 
 	if( storage==LIS_MATRIX_BSR || storage==LIS_MATRIX_VBR )
 	{
@@ -132,21 +139,139 @@ LIS_INT lis_precon_create_iluk(LIS_SOLVER solver, LIS_PRECON precon)
     return LIS_SUCCESS;
 }
 
+/*NEH support for extended "solve_kernel" workflow*/
+#undef __FUNC__
+#define __FUNC__ "lis_precon_psd_create_iluk"
+LIS_INT lis_precon_psd_create_iluk(LIS_SOLVER solver, LIS_PRECON precon)
+{
+	LIS_INT	storage,err;
+	LIS_MATRIX A,B;
+
+	LIS_DEBUG_FUNC_IN;
+
+	storage     = solver->options[LIS_OPTIONS_STORAGE];
+
+	if( storage==LIS_MATRIX_BSR || storage==LIS_MATRIX_VBR )
+	{
+		if( solver->A->matrix_type!=storage )
+		{
+			err = lis_matrix_convert_self(solver);
+			if( err ) return err;
+		}
+	}
+
+	switch( solver->A->matrix_type )
+	{
+	case LIS_MATRIX_CSR:
+		err = lis_symbolic_fact_csr(solver,precon);
+		if( err ) return err;
+        lis_psolve_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolve_iluk_csr;
+        lis_psolvet_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolvet_iluk_csr;
+        precon->is_copy = LIS_TRUE;
+		break;
+	case LIS_MATRIX_BSR:
+/*        err = lis_symbolic_fact_bsr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        err = lis_numerical_fact_bsr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        lis_psolve_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolve_iluk_bsr;*/
+/*        lis_psolvet_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolvet_iluk_bsr;*/
+/*        break;*/
+        err = LIS_ERR_NOT_IMPLEMENTED;
+		return err;
+	case LIS_MATRIX_VBR:
+/*        err = lis_symbolic_fact_vbr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        err = lis_numerical_fact_vbr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        lis_psolve_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolve_iluk_vbr;*/
+/*		lis_psolvet_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolvet_iluk_vbr;*/
+/*        break;*/
+        err = LIS_ERR_NOT_IMPLEMENTED;
+		return err;
+	default:
+		A = solver->A;
+		err = lis_matrix_duplicate(A,&B);
+		if( err ) return err;
+		lis_matrix_set_type(B,LIS_MATRIX_CSR);
+		err = lis_matrix_convert(A,B);
+		if( err ) return err;
+		solver->A = B;
+		err = lis_symbolic_fact_csr(solver,precon);
+		if( err ) return err;
+		lis_psolve_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolve_iluk_csr;
+		lis_psolvet_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolvet_iluk_csr;
+		lis_matrix_destroy(B);
+		solver->A = A;
+		precon->is_copy = LIS_TRUE;
+		break;
+	}
+
+	LIS_DEBUG_FUNC_OUT;
+    return LIS_SUCCESS;
+}
+
+/*NEH support for extended "solve_kernel" workflow*/
+#undef __FUNC__
+#define __FUNC__ "lis_precon_psd_update_iluk"
+LIS_INT lis_precon_psd_update_iluk(LIS_SOLVER solver, LIS_PRECON precon)
+{
+/*    LIS_INT	storage,err;*/
+	LIS_INT	err;
+	LIS_MATRIX A;
+
+	LIS_DEBUG_FUNC_IN;
+
+	switch( solver->A->matrix_type )
+	{
+	case LIS_MATRIX_CSR:
+		err = lis_numerical_fact_csr(solver,precon);
+		if( err ) return err;
+		break;
+	case LIS_MATRIX_BSR:
+/*        err = lis_symbolic_fact_bsr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        err = lis_numerical_fact_bsr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        lis_psolve_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolve_iluk_bsr;*/
+/*        lis_psolvet_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolvet_iluk_bsr;*/
+/*        break;*/
+        err = LIS_ERR_NOT_IMPLEMENTED;
+		return err;
+	case LIS_MATRIX_VBR:
+/*        err = lis_symbolic_fact_vbr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        err = lis_numerical_fact_vbr(solver,precon);*/
+/*        if( err ) return err;*/
+/*        lis_psolve_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolve_iluk_vbr;*/
+/*		lis_psolvet_xxx[LIS_PRECON_TYPE_ILU]  = lis_psolvet_iluk_vbr;*/
+/*        break;*/
+        err = LIS_ERR_NOT_IMPLEMENTED;
+		return err;
+	default:
+/*        nothing should ever really happen here, as any undetermined matrix_type*/
+/*        should have been handled in "psd_create"*/
+		break;
+	}
+
+	LIS_DEBUG_FUNC_OUT;
+    return LIS_SUCCESS;
+}
 
 #undef __FUNC__
 #define __FUNC__ "lis_symbolic_fact_csr"
 LIS_INT lis_symbolic_fact_csr(LIS_SOLVER solver, LIS_PRECON precon)
 {
 #ifdef _OPENMP
-	LIS_INT				err;
-	LIS_INT				i,j,k;
-	LIS_INT				n,levfill;
-	LIS_INT				col,ip,it,jpiv,incl,incu,jmin,kmin;
-	LIS_INT				*levls,*jbuf,*iw,**ulvl;
-	LIS_INT				is,ie,my_rank,nprocs;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
-	LIS_VECTOR		D;
+	LIS_INT	err;
+	LIS_INT	i,j,k;
+	LIS_INT	n,levfill;
+	LIS_INT	col,ip,it,jpiv,incl,incu,jmin,kmin;
+	LIS_INT	*levls,*jbuf,*iw,**ulvl;
+	LIS_INT	is,ie,my_rank,nprocs;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
+	LIS_VECTOR D;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -327,14 +452,14 @@ LIS_INT lis_symbolic_fact_csr(LIS_SOLVER solver, LIS_PRECON precon)
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 #else
-	LIS_INT				err;
-	LIS_INT				i,j,k;
-	LIS_INT				n,levfill;
-	LIS_INT				col,ip,it,jpiv,incl,incu,jmin,kmin;
-	LIS_INT				*levls,*jbuf,*iw,**ulvl;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
-	LIS_VECTOR		D;
+	LIS_INT	err;
+	LIS_INT	i,j,k;
+	LIS_INT	n,levfill;
+	LIS_INT	col,ip,it,jpiv,incl,incu,jmin,kmin;
+	LIS_INT	*levls,*jbuf,*iw,**ulvl;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
+	LIS_VECTOR D;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -513,15 +638,14 @@ LIS_INT lis_symbolic_fact_csr(LIS_SOLVER solver, LIS_PRECON precon)
 LIS_INT lis_numerical_fact_csr(LIS_SOLVER solver, LIS_PRECON precon)
 {
 #ifdef _OPENMP
-	LIS_INT				err;
-	LIS_INT				i,j,k;
-	LIS_INT				n;
-	LIS_INT				col,jpos,jrow;
-	LIS_INT				*jw;
-	LIS_INT				is,ie,my_rank,nprocs;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
-	LIS_VECTOR		D;
+	LIS_INT	i,j,k;
+	LIS_INT	n;
+	LIS_INT	col,jpos,jrow;
+	LIS_INT	*jw;
+	LIS_INT	is,ie,my_rank,nprocs;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
+	LIS_VECTOR D;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -637,13 +761,13 @@ LIS_INT lis_numerical_fact_csr(LIS_SOLVER solver, LIS_PRECON precon)
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 #else
-	LIS_INT				i,j,k;
-	LIS_INT				n;
-	LIS_INT				col,jpos,jrow;
-	LIS_INT				*jw;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
-	LIS_VECTOR		D;
+	LIS_INT	i,j,k;
+	LIS_INT	n;
+	LIS_INT	col,jpos,jrow;
+	LIS_INT	*jw;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
+	LIS_VECTOR D;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -758,10 +882,10 @@ LIS_INT lis_psolve_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 #ifdef _OPENMP
 	LIS_INT i,j,jj,n;
 	LIS_INT is,ie,my_rank,nprocs;
-	LIS_SCALAR *b,*x;
+	LIS_SCALAR *x;
 	LIS_MATRIX_ILU L,U;
 	LIS_VECTOR D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 	LIS_QUAD_DECLAR;
 	#ifdef USE_QUAD_PRECISION
 		LIS_SCALAR *xl;
@@ -773,7 +897,6 @@ LIS_INT lis_psolve_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	L = precon->L;
 	U = precon->U;
 	D = precon->D;
-	b = B->value;
 	x = X->value;
 	#ifdef USE_QUAD_PRECISION
 		xl = X->value_lo;
@@ -867,7 +990,7 @@ LIS_INT lis_psolve_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	LIS_SCALAR *b,*x;
 	LIS_MATRIX_ILU L,U;
 	LIS_VECTOR D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 	LIS_QUAD_DECLAR;
 	#ifdef USE_QUAD_PRECISION
 		LIS_SCALAR *xl;
@@ -965,10 +1088,10 @@ LIS_INT lis_psolvet_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 #ifdef _OPENMP
 	LIS_INT i,j,jj,n;
 	LIS_INT is,ie,my_rank,nprocs;
-	LIS_SCALAR *b,*x;
+	LIS_SCALAR *x;
 	LIS_MATRIX_ILU L,U;
 	LIS_VECTOR D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 	LIS_QUAD_DECLAR;
 	#ifdef USE_QUAD_PRECISION
 		LIS_SCALAR *xl;
@@ -980,7 +1103,6 @@ LIS_INT lis_psolvet_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	L = precon->L;
 	U = precon->U;
 	D = precon->D;
-	b = B->value;
 	x = X->value;
 	#ifdef USE_QUAD_PRECISION
 		xl = X->value_lo;
@@ -1074,7 +1196,7 @@ LIS_INT lis_psolvet_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	LIS_SCALAR *b,*x;
 	LIS_MATRIX_ILU L,U;
 	LIS_VECTOR D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 	LIS_QUAD_DECLAR;
 	#ifdef USE_QUAD_PRECISION
 		LIS_SCALAR *xl;
@@ -1167,14 +1289,14 @@ LIS_INT lis_psolvet_iluk_csr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 LIS_INT lis_symbolic_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 {
 #ifdef _OPENMP
-	LIS_INT				err;
-	LIS_INT				i,j,k,bnr,bs;
-	LIS_INT				n,nr,levfill;
-	LIS_INT				col,ip,it,jpiv,incl,incu,jmin,kmin;
-	LIS_INT				*levls,*jbuf,*iw,**ulvl;
-	LIS_INT				is,ie,my_rank,nprocs;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	err;
+	LIS_INT	i,j,k,bnr,bs;
+	LIS_INT	n,nr,levfill;
+	LIS_INT	col,ip,it,jpiv,incl,incu,jmin,kmin;
+	LIS_INT	*levls,*jbuf,*iw,**ulvl;
+	LIS_INT	is,ie,my_rank,nprocs;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -1359,13 +1481,13 @@ LIS_INT lis_symbolic_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 #else
-	LIS_INT				err;
-	LIS_INT				i,j,k,bnr,bs;
-	LIS_INT				n,nr,levfill;
-	LIS_INT				col,ip,it,jpiv,incl,incu,jmin,kmin;
-	LIS_INT				*levls,*jbuf,*iw,**ulvl;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	err;
+	LIS_INT	i,j,k,bnr,bs;
+	LIS_INT	n,nr,levfill;
+	LIS_INT	col,ip,it,jpiv,incl,incu,jmin,kmin;
+	LIS_INT	*levls,*jbuf,*iw,**ulvl;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -1549,15 +1671,14 @@ LIS_INT lis_symbolic_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 LIS_INT lis_numerical_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 {
 #ifdef _OPENMP
-	LIS_INT				err;
-	LIS_INT				i,j,k,bnr,bs;
-	LIS_INT				n,nr;
-	LIS_INT				col,jpos,jrow;
-	LIS_INT				*jw;
-	LIS_INT				is,ie,my_rank,nprocs;
-	LIS_SCALAR		buf[16];
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	i,j,k,bnr,bs;
+	LIS_INT	n,nr;
+	LIS_INT	col,jpos,jrow;
+	LIS_INT	*jw;
+	LIS_INT	is,ie,my_rank,nprocs;
+	LIS_SCALAR buf[16];
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -1694,7 +1815,7 @@ LIS_INT lis_numerical_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 					break;
 				}
 			}
-			lis_array_invGauss(bnr,&D->value[bs*i]);
+			lis_array_ge(bnr,&D->value[bs*i]);
 		}
 	}
 	lis_free(jw);
@@ -1702,13 +1823,13 @@ LIS_INT lis_numerical_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 #else
-	LIS_INT				i,j,k,bnr,bs;
-	LIS_INT				n,nr;
-	LIS_INT				col,jpos,jrow;
-	LIS_INT				*jw;
-	LIS_SCALAR		buf[16];
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	i,j,k,bnr,bs;
+	LIS_INT	n,nr;
+	LIS_INT	col,jpos,jrow;
+	LIS_INT	*jw;
+	LIS_SCALAR buf[16];
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -1836,7 +1957,7 @@ LIS_INT lis_numerical_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 				break;
 			}
 		}
-		lis_array_invGauss(bnr,&D->value[bs*i]);
+		lis_array_ge(bnr,&D->value[bs*i]);
 	}
 	lis_free(jw);
 
@@ -1851,13 +1972,13 @@ LIS_INT lis_numerical_fact_bsr(LIS_SOLVER solver, LIS_PRECON precon)
 LIS_INT lis_psolve_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 {
 #ifdef _OPENMP
-	LIS_INT i,j,jj,nr,bnr,ii,bs;
+	LIS_INT i,j,jj,nr,bnr,bs;
 	LIS_INT is,ie,my_rank,nprocs;
-	LIS_SCALAR w[3],t;
-	LIS_SCALAR *b,*x;
+	LIS_SCALAR w[3];
+	LIS_SCALAR *x;
 	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 
 	/*
 	 *  LUx = b
@@ -1870,7 +1991,6 @@ LIS_INT lis_psolve_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	L = precon->L;
 	U = precon->U;
 	D = precon->WD;
-	b = B->value;
 	x = X->value;
 	nr = solver->A->nr;
 	bnr = solver->A->bnr;
@@ -1925,7 +2045,7 @@ LIS_INT lis_psolve_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	LIS_SCALAR *b,*x;
 	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 
 	/*
 	 *  LUx = b
@@ -1976,13 +2096,13 @@ LIS_INT lis_psolve_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 LIS_INT lis_psolvet_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 {
 #ifdef _OPENMP
-	LIS_INT i,j,jj,nr,bnr,ii,bs;
+	LIS_INT i,j,jj,nr,bnr,bs;
 	LIS_INT is,ie,my_rank,nprocs;
-	LIS_SCALAR w[3],t;
-	LIS_SCALAR *b,*x;
+	LIS_SCALAR w[3];
+	LIS_SCALAR *x;
 	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 
 	/*
 	 *  LUx = b
@@ -1995,7 +2115,6 @@ LIS_INT lis_psolvet_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	L = precon->L;
 	U = precon->U;
 	D = precon->WD;
-	b = B->value;
 	x = X->value;
 	nr = solver->A->nr;
 	bnr = solver->A->bnr;
@@ -2050,7 +2169,7 @@ LIS_INT lis_psolvet_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	LIS_SCALAR *b,*x;
 	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 
 	/*
 	 *  LUx = b
@@ -2101,14 +2220,14 @@ LIS_INT lis_psolvet_iluk_bsr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 LIS_INT lis_symbolic_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 {
 #ifdef _OPENMP
-	LIS_INT				err;
-	LIS_INT				i,j,k,kk,t,bnr,bs;
-	LIS_INT				n,nr,levfill;
-	LIS_INT				col,ip,it,jpiv,incl,incu,jmin,kmin;
-	LIS_INT				*levls,*jbuf,*iw,**ulvl;
-	LIS_INT				is,ie,my_rank,nprocs;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	err;
+	LIS_INT	i,j,k,bnr;
+	LIS_INT	n,nr,levfill;
+	LIS_INT	col,ip,it,jpiv,incl,incu,jmin,kmin;
+	LIS_INT	*levls,*jbuf,*iw,**ulvl;
+	LIS_INT	is,ie,my_rank,nprocs;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -2118,7 +2237,6 @@ LIS_INT lis_symbolic_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 	n      = A->n;
 	nr     = A->nr;
 	bnr    = A->bnr;
-	bs     = bnr*bnr;
 	levfill = solver->options[LIS_OPTIONS_FILL];
 	nprocs = omp_get_max_threads();
 
@@ -2307,13 +2425,13 @@ LIS_INT lis_symbolic_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 #else
-	LIS_INT				err;
-	LIS_INT				i,j,k,bnr,bs;
-	LIS_INT				n,nr,levfill;
-	LIS_INT				col,ip,it,jpiv,incl,incu,jmin,kmin;
-	LIS_INT				*levls,*jbuf,*iw,**ulvl;
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	err;
+	LIS_INT	i,j,k,bnr,bs;
+	LIS_INT	n,nr,levfill;
+	LIS_INT	col,ip,it,jpiv,incl,incu,jmin,kmin;
+	LIS_INT	*levls,*jbuf,*iw,**ulvl;
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -2515,15 +2633,14 @@ LIS_INT lis_symbolic_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 {
 #ifdef _OPENMP
-	LIS_INT				err;
-	LIS_INT				i,j,k,dim,sz,mm,nn,kk;
-	LIS_INT				n,nr;
-	LIS_INT				col,jpos,jrow;
-	LIS_INT				*jw,*bsz;
-	LIS_INT				is,ie,my_rank,nprocs;
-	LIS_SCALAR		buf[1024];
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	i,j,k,dim,sz,mm,nn,kk;
+	LIS_INT	n,nr;
+	LIS_INT	col,jpos,jrow;
+	LIS_INT	*jw,*bsz;
+	LIS_INT	is,ie,my_rank,nprocs;
+	LIS_SCALAR buf[1024];
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -2607,7 +2724,7 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 				jrow = L->index[i][j];
 				mm   = dim;
 				nn   = _DIM(bsz,jrow);
-				lis_array_matmat2(mm,nn,nn,L->values[i][j],mm,D->v_value[jrow],nn,buf,mm,LIS_INS_VALUE);
+				lis_array_matmat_ns(mm,nn,nn,L->values[i][j],mm,D->v_value[jrow],nn,buf,mm,LIS_INS_VALUE);
 				memcpy(L->values[i][j],buf,mm*nn*sizeof(LIS_SCALAR));
 
 				for(k=0;k<U->nnz[jrow];k++)
@@ -2618,16 +2735,16 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 					if( col<i )
 					{
 						kk = _DIM(bsz,col);
-						lis_array_matmat2(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,L->values[i][jpos],mm,LIS_SUB_VALUE);
+						lis_array_matmat_ns(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,L->values[i][jpos],mm,LIS_SUB_VALUE);
 					}
 					else if( col==i )
 					{
-						lis_array_matmat2(mm,mm,nn,L->values[i][j],mm,U->values[jrow][k],nn,D->v_value[i],mm,LIS_SUB_VALUE);
+						lis_array_matmat_ns(mm,mm,nn,L->values[i][j],mm,U->values[jrow][k],nn,D->v_value[i],mm,LIS_SUB_VALUE);
 					}
 					else
 					{
 						kk = _DIM(bsz,col);
-						lis_array_matmat2(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,U->values[i][jpos],mm,LIS_SUB_VALUE);
+						lis_array_matmat_ns(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,U->values[i][jpos],mm,LIS_SUB_VALUE);
 					}
 				}
 			}
@@ -2644,7 +2761,7 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 				jw[col] = -1;
 			}
 
-			lis_array_invGauss(dim,D->v_value[i]);
+			lis_array_ge(dim,D->v_value[i]);
 		}
 	}
 	lis_free(jw);
@@ -2652,13 +2769,13 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 #else
-	LIS_INT				i,j,k,dim,sz,mm,nn,kk;
-	LIS_INT				n,nr;
-	LIS_INT				col,jpos,jrow;
-	LIS_INT				*jw,*bsz;
-	LIS_SCALAR		buf[1024];
-	LIS_MATRIX		A;
-	LIS_MATRIX_ILU	L,U;
+	LIS_INT	i,j,k,dim,sz,mm,nn,kk;
+	LIS_INT	n,nr;
+	LIS_INT	col,jpos,jrow;
+	LIS_INT	*jw,*bsz;
+	LIS_SCALAR buf[1024];
+	LIS_MATRIX A;
+	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG	D;
 
 	LIS_DEBUG_FUNC_IN;
@@ -2737,7 +2854,7 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 			mm   = dim;
 			nn   = _DIM(bsz,jrow);
 /*			printf("i=%d j=%d jrow=%d mm=%d nn=%d matmat\n",i,j,jrow,mm,nn);*/
-			lis_array_matmat2(mm,nn,nn,L->values[i][j],mm,D->v_value[jrow],nn,buf,mm,LIS_INS_VALUE);
+			lis_array_matmat_ns(mm,nn,nn,L->values[i][j],mm,D->v_value[jrow],nn,buf,mm,LIS_INS_VALUE);
 			memcpy(L->values[i][j],buf,mm*nn*sizeof(LIS_SCALAR));
 
 			for(k=0;k<U->nnz[jrow];k++)
@@ -2748,16 +2865,16 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 				if( col<i )
 				{
 					kk = _DIM(bsz,col);
-					lis_array_matmat2(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,L->values[i][jpos],mm,LIS_SUB_VALUE);
+					lis_array_matmat_ns(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,L->values[i][jpos],mm,LIS_SUB_VALUE);
 				}
 				else if( col==i )
 				{
-					lis_array_matmat2(mm,mm,nn,L->values[i][j],mm,U->values[jrow][k],nn,D->v_value[i],mm,LIS_SUB_VALUE);
+					lis_array_matmat_ns(mm,mm,nn,L->values[i][j],mm,U->values[jrow][k],nn,D->v_value[i],mm,LIS_SUB_VALUE);
 				}
 				else
 				{
 					kk = _DIM(bsz,col);
-					lis_array_matmat2(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,U->values[i][jpos],mm,LIS_SUB_VALUE);
+					lis_array_matmat_ns(mm,kk,nn,L->values[i][j],mm,U->values[jrow][k],nn,U->values[i][jpos],mm,LIS_SUB_VALUE);
 				}
 			}
 		}
@@ -2774,7 +2891,7 @@ LIS_INT lis_numerical_fact_vbr(LIS_SOLVER solver, LIS_PRECON precon)
 			jw[col] = -1;
 		}
 
-		lis_array_invGauss(dim,D->v_value[i]);
+		lis_array_ge(dim,D->v_value[i]);
 	}
 	lis_free(jw);
 
@@ -2790,11 +2907,11 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 #ifdef _OPENMP
 	LIS_INT i,j,jj,nr,bnr,dim,sz,*bsz;
 	LIS_INT is,ie,my_rank,nprocs;
-	LIS_SCALAR w[1024],t;
-	LIS_SCALAR *b,*x;
+	LIS_SCALAR w[1024];
+	LIS_SCALAR *x;
 	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 
 	/*
 	 *  LUx = b
@@ -2807,7 +2924,6 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	L = precon->L;
 	U = precon->U;
 	D = precon->WD;
-	b = B->value;
 	x = X->value;
 	nr = solver->A->nr;
 	bsz = L->bsz;
@@ -2827,7 +2943,7 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 			{
 				jj     = L->index[i][j];
 				sz     = _DIM(bsz,jj);
-				lis_array_matvec2(dim,sz,L->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
+				lis_array_matvec_ns(dim,sz,L->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
 			}
 		}
 		for(i=nr-1; i>=0; i--)
@@ -2838,9 +2954,9 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 			{
 				jj = U->index[i][j];
 				sz = _DIM(bsz,jj);
-				lis_array_matvec2(dim,sz,U->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
+				lis_array_matvec_ns(dim,sz,U->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
 			}
-			lis_array_matvec2(dim,dim,D->v_value[i],dim,&x[bnr],w,LIS_INS_VALUE);
+			lis_array_matvec_ns(dim,dim,D->v_value[i],dim,&x[bnr],w,LIS_INS_VALUE);
 			memcpy(&x[bnr],w,dim*sizeof(LIS_SCALAR));
 		}
 	}
@@ -2853,7 +2969,7 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 	LIS_SCALAR *b,*x;
 	LIS_MATRIX_ILU L,U;
 	LIS_MATRIX_DIAG D;
-	LIS_PRECON	precon;
+	LIS_PRECON precon;
 
 	/*
 	 *  LUx = b
@@ -2880,7 +2996,7 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 		{
 			jj     = L->index[i][j];
 			sz     = _DIM(bsz,jj);
-			lis_array_matvec2(dim,sz,L->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
+			lis_array_matvec_ns(dim,sz,L->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
 		}
 	}
 	for(i=nr-1; i>=0; i--)
@@ -2891,9 +3007,9 @@ LIS_INT lis_psolve_iluk_vbr(LIS_SOLVER solver, LIS_VECTOR B, LIS_VECTOR X)
 		{
 			jj = U->index[i][j];
 			sz = _DIM(bsz,jj);
-			lis_array_matvec2(dim,sz,U->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
+			lis_array_matvec_ns(dim,sz,U->values[i][j],dim,&x[bsz[jj]],&x[bnr],LIS_SUB_VALUE);
 		}
-		lis_array_matvec2(dim,dim,D->v_value[i],dim,&x[bnr],w,LIS_INS_VALUE);
+		lis_array_matvec_ns(dim,dim,D->v_value[i],dim,&x[bnr],w,LIS_INS_VALUE);
 		memcpy(&x[bnr],w,dim*sizeof(LIS_SCALAR));
 	}
 
