@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -39,6 +39,9 @@
 #endif
 #include <string.h>
 #include <stdarg.h>
+/*NEH deleted reference to "math.h" from lis.h, and placed here,*/
+/*NEH for consistency . . .*/
+#include <math.h>
 #include <ctype.h>
 #ifdef _OPENMP
 	#include <omp.h>
@@ -141,7 +144,7 @@ LIS_INT LIS_USE_AT_TYPE[] = {
 	0,
 	LIS_MATRIX_CSC,LIS_MATRIX_CSR
 	};
-#define LIS_SOLVER_OPTION_LEN	46
+#define LIS_SOLVER_OPTION_LEN		46
 #define LIS_PRINT_LEN			4
 #define LIS_SCALE_LEN			3
 #define LIS_TRUEFALSE_LEN		2
@@ -204,16 +207,16 @@ LIS_INT lis_solver_init(LIS_SOLVER solver)
 	solver->x        = NULL;
 	solver->d        = NULL;
 	solver->work     = NULL;
-	solver->residual = NULL;
+	solver->rhistory = NULL;
 	solver->precon   = NULL;
 
 	solver->worklen   = 0;
 	solver->iter      = 0;
 	solver->iter2     = 0;
 	solver->resid     = 0;
-	solver->times     = 0;
-	solver->itimes    = 0;
-	solver->ptimes    = 0;
+	solver->time      = 0;
+	solver->itime     = 0;
+	solver->ptime     = 0;
 	solver->precision = LIS_PRECISION_DOUBLE;
 
 	solver->options[LIS_OPTIONS_SOLVER]               = LIS_SOLVER_BICG;
@@ -315,7 +318,7 @@ LIS_INT lis_solver_destroy(LIS_SOLVER solver)
 		lis_solver_work_destroy(solver);
 		lis_vector_destroy(solver->d);
 		if( solver->At ) lis_matrix_destroy(solver->At);
-		if( solver->residual ) lis_free(solver->residual);
+		if( solver->rhistory ) lis_free(solver->rhistory);
 		lis_free(solver);
 	}
 
@@ -323,13 +326,25 @@ LIS_INT lis_solver_destroy(LIS_SOLVER solver)
 	return LIS_SUCCESS;
 }
 
+/*NEH support for extended "solve_kernel" workflow*/
+#undef __FUNC__
+#define __FUNC__ "lis_solver_set_matrix"
+LIS_INT lis_solver_set_matrix(LIS_MATRIX A, LIS_SOLVER solver)
+{
+	LIS_DEBUG_FUNC_IN;
+
+	solver->A = A;
+
+	LIS_DEBUG_FUNC_OUT;
+	return LIS_SUCCESS;
+}
 
 #undef __FUNC__
 #define __FUNC__ "lis_solve"
 LIS_INT lis_solve(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER solver)
 {
-        LIS_INT		err;
-	LIS_PRECON      precon;
+    LIS_INT	err;
+	LIS_PRECON precon;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -351,7 +366,7 @@ LIS_INT lis_solve(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER solver)
 		return err;
 	}
 
-	/* Core Kernel of lis_solve() */
+	/* core kernel of lis_solve() */
 	lis_solve_kernel(A, b, x, solver, precon);
 
 	lis_precon_destroy(precon);
@@ -364,18 +379,18 @@ LIS_INT lis_solve(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER solver)
 #define __FUNC__ "lis_solve_kernel"
 LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER solver, LIS_PRECON precon)
 {
-	LIS_INT			nsolver, precon_type, maxiter;
-	LIS_INT			err;
-	LIS_SCALAR	*residual;
-	LIS_VECTOR	xx;
+	LIS_INT	nsolver, precon_type, maxiter;
+	LIS_INT	err;
+	LIS_SCALAR *rhistory;
+	LIS_VECTOR xx;
 
 	LIS_INT output;
 	LIS_INT scale;
 	LIS_INT conv_cond;
 	LIS_INT precision,is_use_at,storage,block;
-	LIS_INT i,n,np;
-	double p_c_times, p_i_times,itimes;
-	LIS_SCALAR nrm2,tol,tol_w;
+	LIS_INT i,n;
+	double p_c_time, p_i_time,itime;
+	LIS_REAL nrm2,tol,tol_w;
 	LIS_VECTOR t;
 	LIS_VECTOR bb;
 	LIS_MATRIX AA,B;
@@ -468,7 +483,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 	}
 	if( solver->options[LIS_OPTIONS_INITGUESS_ZEROS] )
 	{
-	  if( output ) lis_printf(A->comm,"initial vector x = 0\n");
+	  if( output ) lis_printf(A->comm,"initial vector x      : 0\n");
 		#ifndef USE_QUAD_PRECISION
 			lis_vector_set_all(0.0,xx);
 		#else
@@ -484,7 +499,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 	}
 	else
 	{
-	  if( output ) lis_printf(A->comm,"initial vector x = user defined\n"); 
+	  if( output ) lis_printf(A->comm,"initial vector x      : user defined\n"); 
 		#ifndef USE_QUAD_PRECISION
 			lis_vector_copy(x,xx);
 		#else
@@ -500,25 +515,24 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 	}
 
 	/* create residual history vector */
-	if( solver->residual ) lis_free(solver->residual);
-	residual = (LIS_SCALAR *)lis_malloc((maxiter+2)*sizeof(LIS_SCALAR),"lis_solve::residual");
-	if( residual==NULL )
+	if( solver->rhistory ) lis_free(solver->rhistory);
+	rhistory = (LIS_SCALAR *)lis_malloc((maxiter+2)*sizeof(LIS_SCALAR),"lis_solve::rhistory");
+	if( rhistory==NULL )
 	{
 		LIS_SETERR_MEM((maxiter+2)*sizeof(LIS_SCALAR));
 		lis_vector_destroy(xx);
 		solver->retcode = err;
 		return err;
 	}
-	residual[0] = 1.0;
+	rhistory[0] = 1.0;
 
 
 	n       = A->n;
-	np      = A->np;
 	t       = NULL;
 	At      = NULL;
 
 
-	p_c_times = lis_wtime();
+	p_c_time = lis_wtime();
 	if( precon_type==LIS_PRECON_TYPE_IS )
 	{
 		if( solver->d==NULL )
@@ -531,7 +545,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 		}
 		if( !A->is_scaled )
 		{
-			lis_matrix_scaling(A,b,solver->d,LIS_SCALE_JACOBI);
+			lis_matrix_scale(A,b,solver->d,LIS_SCALE_JACOBI);
 		}
 		else if( !b->is_scaled )
 		{
@@ -560,7 +574,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 		}
 		if( !A->is_scaled )
 		{
-			lis_matrix_scaling(A,b,solver->d,LIS_SCALE_JACOBI);
+			lis_matrix_scale(A,b,solver->d,LIS_SCALE_JACOBI);
 		}
 	}
 	else if( scale )
@@ -591,7 +605,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 			if( err ) return err;
 			lis_matrix_diag_copy(A->D,solver->WD);
 			lis_matrix_diag_inverse(solver->WD);
-			lis_matrix_bscaling_bsr(A,solver->WD);
+			lis_matrix_bscale_bsr(A,solver->WD);
 			lis_vector_duplicate(A,&t);
 			lis_matrix_diag_matvec(solver->WD,b,t);
 			lis_vector_copy(t,b);
@@ -614,7 +628,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 			}
 			if( !A->is_scaled )
 			{
-				lis_matrix_scaling(A,b,solver->d,scale);
+				lis_matrix_scale(A,b,solver->d,scale);
 			}
 			else if( !b->is_scaled )
 			{
@@ -649,10 +663,10 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 		bb = b;
 	}
 
-	p_c_times = lis_wtime() - p_c_times;
-	itimes = lis_wtime();
+	p_c_time = lis_wtime() - p_c_time;
+	itime = lis_wtime();
 
-	/* Matrix Convert */
+	/* convert matrix */
 	solver->A  = AA;
 	solver->b  = bb;
 	err = lis_matrix_convert_self(solver);
@@ -660,7 +674,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 	{
 		lis_vector_destroy(xx);
 		lis_solver_work_destroy(solver);
-		lis_free(residual);
+		lis_free(rhistory);
 		solver->retcode = err;
 		return err;
 	}
@@ -668,15 +682,15 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 
 	if( A->my_rank==0 )
 	{
-#ifdef _LONG_DOUBLE
-	  if( output ) printf("precision : long double\n"); 
+#ifdef _LONG__DOUBLE
+	  if( output ) printf("precision             : long double\n");
 #else
-	  if( output ) printf("precision : %s\n", lis_precisionname[precision]); 
+ 	  if( output ) printf("precision             : %s\n", lis_precisionname[precision]); 
 #endif
-#ifdef _LONGLONG
-	  if( output ) printf("solver    : %s %lld\n", lis_solvername[nsolver],nsolver); 
+#ifdef _LONG__LONG
+	  if( output ) printf("linear solver         : %s\n", lis_solvername[nsolver]); 
 #else
-	  if( output ) printf("solver    : %s %d\n", lis_solvername[nsolver],nsolver); 
+	  if( output ) printf("linear solver         : %s\n", lis_solvername[nsolver]); 
 #endif
 		switch( precon_type )
 		{
@@ -684,7 +698,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 			i = solver->options[LIS_OPTIONS_FILL];
 			if( A->matrix_type==LIS_MATRIX_BSR || A->matrix_type==LIS_MATRIX_VBR )
 			{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 			  if( output ) sprintf(buf,"Block %s(%lld)",lis_preconname[precon_type],i); 
 #else
 			  if( output ) sprintf(buf,"Block %s(%d)",lis_preconname[precon_type],i); 
@@ -692,7 +706,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 			}
 			else
 			{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 			  if( output ) sprintf(buf,"%s(%lld)",lis_preconname[precon_type],i); 
 #else
 			  if( output ) sprintf(buf,"%s(%d)",lis_preconname[precon_type],i); 
@@ -705,20 +719,35 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 		}
 		if( solver->options[LIS_OPTIONS_ADDS] && precon_type )
 		{
-		  if( output ) printf("precon    : %s + additive schwarz\n", buf); 
+ 		  if( output ) printf("preconditioner        : %s + Additive Schwarz\n", buf);
 		}
 		else
 		{
-		  if( output ) printf("precon    : %s\n", buf); 
+		  if( output ) printf("preconditioner        : %s\n", buf); 
 		}
 	}
 	switch(conv_cond)
 	{
 	case LIS_CONV_COND_NRM2_R:
-	case LIS_CONV_COND_NRM2_B:
 		if( A->my_rank==0 )
 		{
-		  if( output ) ("CONV_COND : ||r||_2 <= %6.1e*||r_0||_2\n", tol); 
+#ifdef _LONG__DOUBLE
+		  if( output ) printf("convergence condition : ||b-Ax||_2 <= %6.1Le * ||b-Ax_0||_2\n", tol); 		  
+#else 
+		  if( output ) printf("convergence condition : ||b-Ax||_2 <= %6.1e * ||b-Ax_0||_2\n", tol); 		  
+#endif
+		}
+		break;		
+	case LIS_CONV_COND_NRM2_B:
+		lis_vector_nrm2(b,&nrm2);
+		nrm2 = nrm2*tol;
+		if( A->my_rank==0 )
+		{
+#ifdef _LONG__DOUBLE
+		  if( output ) printf("convergence condition : ||b-Ax||_2 <= %6.1Le*||b||_2 = %6.1Le\n", tol,nrm2);
+#else
+		  if( output ) printf("convergence condition : ||b-Ax||_2 <= %6.1e*||b||_2 = %6.1e\n", tol,nrm2);
+#endif
 		}
 		break;
 	case LIS_CONV_COND_NRM1_B:
@@ -726,7 +755,11 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 		nrm2 = nrm2*tol_w + tol;
 		if( A->my_rank==0 )
 		{
-		  if( output ) printf("conv_cond : ||r||_1 <= %6.1e*||b||_1 + %6.1e = %6.1e\n", tol_w,tol,nrm2);
+#ifdef _LONG__DOUBLE
+		  if( output ) printf("convergence condition : ||b-Ax||_1 <= %6.1Le*||b||_1 + %6.1Le = %6.1Le\n", tol_w,tol,nrm2);
+#else
+		  if( output ) printf("convergence condition : ||b-Ax||_1 <= %6.1e*||b||_1 + %6.1e = %6.1e\n", tol_w,tol,nrm2);
+#endif
 		}
 		break;
 	}
@@ -734,15 +767,15 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 	{
 		if( AA->matrix_type==LIS_MATRIX_BSR || AA->matrix_type==LIS_MATRIX_BSC )
 		{
-#ifdef _LONGLONG
-		  if( output ) printf("storage   : %s(%lld x %lld)\n", lis_storagename[AA->matrix_type-1],block,block); 
+#ifdef _LONG__LONG
+		  if( output ) printf("matrix storage format : %s(%lld x %lld)\n", lis_storagename[AA->matrix_type-1],block,block);
 #else
-		  if( output ) printf("storage   : %s(%d x %d)\n", lis_storagename[AA->matrix_type-1],block,block); 
+		  if( output ) printf("matrix storage format : %s(%d x %d)\n", lis_storagename[AA->matrix_type-1],block,block); 
 #endif
 		}
 		else
 		{
-		  if( output ) printf("storage   : %s\n", lis_storagename[AA->matrix_type-1]); 
+		  if( output ) printf("matrix storage format : %s\n", lis_storagename[AA->matrix_type-1]); 
 		}
 	}
 
@@ -768,7 +801,7 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 	solver->x        = xx;
 	solver->xx       = x;
 	solver->precon   = precon;
-	solver->residual = residual;
+	solver->rhistory = rhistory;
 
 	/* execute solver */
 	#ifndef USE_QUAD_PRECISION
@@ -814,13 +847,13 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 			}
 		#endif
 	}
-	itimes = lis_wtime() - itimes - solver->ptimes;
-	p_i_times = solver->ptimes;
-	solver->ptimes = p_c_times + p_i_times;
-	solver->p_c_times = p_c_times;
-	solver->p_i_times = p_i_times;
-	solver->times  = solver->ptimes + itimes;
-	solver->itimes = itimes;
+	itime = lis_wtime() - itime - solver->ptime;
+	p_i_time = solver->ptime;
+	solver->ptime = p_c_time + p_i_time;
+	solver->p_c_time = p_c_time;
+	solver->p_i_time = p_i_time;
+	solver->time  = solver->ptime + itime;
+	solver->itime = itime;
 	lis_solver_work_destroy(solver);
 	lis_vector_duplicate(A,&t);
 	xx->precision = LIS_PRECISION_DEFAULT;
@@ -837,22 +870,21 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 		}
 	}
 	lis_vector_nrm2(t,&nrm2);
-	/*
-	solver->resid = nrm2;
-	*/
+
+	/* solver->resid = nrm2; */
 	if( A->my_rank==0 )
 	{
 		if( err )
 		{
-#ifdef _LONGLONG
-		  if( output ) printf("lis_solve : %s(code=%lld)\n\n",lis_returncode[err],err); 
+#ifdef _LONG__LONG
+		  if( output ) printf("linear solvef status  : %s(code=%lld)\n\n",lis_returncode[err],err);
 #else
-		  if( output ) printf("lis_solve : %s(code=%d)\n\n",lis_returncode[err],err); 
+		  if( output ) printf("linear solver status  : %s(code=%d)\n\n",lis_returncode[err],err); 
 #endif
 		}
 		else
 		{
-		  if( output ) printf("lis_solve : normal end\n\n"); 
+		  if( output ) printf("linear solver status  : normal end\n\n"); 
 		}
 	}
 	if( precision==LIS_PRECISION_DOUBLE )
@@ -866,7 +898,8 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 
 
 	lis_vector_destroy(t);
-/*	lis_vector_destroy(d);*/
+
+	/* lis_vector_destroy(d); */
 	lis_vector_destroy(xx);
 
 	LIS_DEBUG_FUNC_OUT;
@@ -875,23 +908,22 @@ LIS_INT lis_solve_kernel(LIS_MATRIX A, LIS_VECTOR b, LIS_VECTOR x, LIS_SOLVER so
 
 #undef __FUNC__
 #define __FUNC__ "lis_solver_get_initial_residual"
-LIS_INT lis_solver_get_initial_residual(LIS_SOLVER solver, LIS_PRECON M, LIS_VECTOR t, LIS_VECTOR r, LIS_SCALAR *bnrm2)
+LIS_INT lis_solver_get_initial_residual(LIS_SOLVER solver, LIS_PRECON M, LIS_VECTOR t, LIS_VECTOR r, LIS_REAL *bnrm2)
 {
-	LIS_INT			output,conv;
+	LIS_INT	output,conv;
 	#ifdef USE_QUAD_PRECISION
 		LIS_INT	i;
 	#endif
-	LIS_MATRIX	A;
-	LIS_VECTOR	x,b,p,xx;
-	LIS_SCALAR	nrm2;
-	LIS_REAL	tol,tol_w,tol_switch;
-
+	LIS_MATRIX A;
+	LIS_VECTOR x,xx,b,p;
+	LIS_REAL nrm2;
+	LIS_REAL tol,tol_w,tol_switch;
+	
 	LIS_DEBUG_FUNC_IN;
 
 	A  = solver->A;
 	b  = solver->b;
 	x  = solver->x;
-	xx = solver->xx;
 	output     = solver->options[LIS_OPTIONS_OUTPUT];
 	conv       = solver->options[LIS_OPTIONS_CONV_COND];
 	tol        = solver->params[LIS_PARAMS_RESID-LIS_OPTIONS_LEN];
@@ -899,7 +931,7 @@ LIS_INT lis_solver_get_initial_residual(LIS_SOLVER solver, LIS_PRECON M, LIS_VEC
 	tol_switch = solver->params[LIS_PARAMS_SWITCH_RESID-LIS_OPTIONS_LEN];
 
 
-	/* Initial Residual */
+	/* initial residual */
 	if( M==NULL )
 	{
 		p = r;
@@ -922,7 +954,7 @@ LIS_INT lis_solver_get_initial_residual(LIS_SOLVER solver, LIS_PRECON M, LIS_VEC
 			}
 			else
 			{
-				lis_matvec(A,xx,p);           /* p = Ax    */
+				lis_matvec(A,xx,p);          /* p = Ax    */
 				lis_vector_xpay(b,-1,p);     /* p = b - p */
 				
 				#ifdef _OPENMP
@@ -986,10 +1018,14 @@ LIS_INT lis_solver_get_initial_residual(LIS_SOLVER solver, LIS_PRECON M, LIS_VEC
 
 	if( output && (r->precision==LIS_PRECISION_QUAD && solver->precision!=LIS_PRECISION_SWITCH) )
 	{
-		if( output & LIS_PRINT_MEM ) solver->residual[0] = nrm2;
-		if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", 0, nrm2); 
+		if( output & LIS_PRINT_MEM ) solver->rhistory[0] = nrm2;
+#ifdef _LONG__DOUBLE
+		if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iteration: %5d  relative residual = %Le\n", 0, nrm2); 
+#else
+		if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iteration: %5d  relative residual = %e\n", 0, nrm2); 
+#endif
 	}
-	if( nrm2 <= solver->params[LIS_PARAMS_RESID-LIS_OPTIONS_LEN] )
+	if( nrm2 <= fabs(solver->params[LIS_PARAMS_RESID-LIS_OPTIONS_LEN]) )
 	{
 		solver->retcode = LIS_SUCCESS;
 		solver->iter    = 1;
@@ -1012,7 +1048,7 @@ LIS_INT lis_solver_get_initial_residual(LIS_SOLVER solver, LIS_PRECON M, LIS_VEC
 #define __FUNC__ "lis_solver_set_optionC"
 LIS_INT lis_solver_set_optionC(LIS_SOLVER solver)
 {
-	LIS_ARGS	p;
+	LIS_ARGS p;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -1031,7 +1067,7 @@ LIS_INT lis_solver_set_optionC(LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option"
 LIS_INT lis_solver_set_option(char *text, LIS_SOLVER solver)
 {
-	LIS_ARGS	args,p;
+	LIS_ARGS args,p;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -1114,7 +1150,7 @@ LIS_INT lis_solver_set_option2(char* arg1, char *arg2, LIS_SOLVER solver)
 			default:
 				if( LIS_SOLVER_OPTACT[i] < LIS_OPTIONS_LEN )
 				{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 					sscanf(arg2, "%lld", &solver->options[LIS_SOLVER_OPTACT[i]]);
 #else
 					sscanf(arg2, "%d", &solver->options[LIS_SOLVER_OPTACT[i]]);
@@ -1122,7 +1158,7 @@ LIS_INT lis_solver_set_option2(char* arg1, char *arg2, LIS_SOLVER solver)
 				}
 				else
 				{
-#ifdef _LONG_DOUBLE
+#ifdef _LONG__DOUBLE
 					sscanf(arg2, "%Lg", &solver->params[LIS_SOLVER_OPTACT[i]-LIS_OPTIONS_LEN]);
 #else
 					sscanf(arg2, "%lg", &solver->params[LIS_SOLVER_OPTACT[i]-LIS_OPTIONS_LEN]);
@@ -1140,13 +1176,13 @@ LIS_INT lis_solver_set_option2(char* arg1, char *arg2, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_solver"
 LIS_INT lis_solver_set_option_solver(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='9' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_SOLVER]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_SOLVER]);
@@ -1171,13 +1207,13 @@ LIS_INT lis_solver_set_option_solver(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_psolver"
 LIS_INT lis_solver_set_option_psolver(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='9' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_PSOLVER]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_PSOLVER]);
@@ -1202,13 +1238,13 @@ LIS_INT lis_solver_set_option_psolver(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_precon"
 LIS_INT lis_solver_set_option_precon(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='9' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_PRECON]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_PRECON]);
@@ -1243,13 +1279,13 @@ LIS_INT lis_solver_set_option_precon(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_pprecon"
 LIS_INT lis_solver_set_option_pprecon(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='9' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_PPRECON]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_PPRECON]);
@@ -1284,13 +1320,13 @@ LIS_INT lis_solver_set_option_pprecon(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_print"
 LIS_INT lis_solver_set_option_print(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='3' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_OUTPUT]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_OUTPUT]);
@@ -1315,13 +1351,13 @@ LIS_INT lis_solver_set_option_print(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_scale"
 LIS_INT lis_solver_set_option_scale(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='2' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_SCALE]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_SCALE]);
@@ -1346,13 +1382,13 @@ LIS_INT lis_solver_set_option_scale(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_truefalse"
 LIS_INT lis_solver_set_option_truefalse(char *argv, LIS_INT opt, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='1' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[opt]);
 #else
 		sscanf(argv, "%d", &solver->options[opt]);
@@ -1377,13 +1413,13 @@ LIS_INT lis_solver_set_option_truefalse(char *argv, LIS_INT opt, LIS_SOLVER solv
 #define __FUNC__ "lis_solver_set_option_precision"
 LIS_INT lis_solver_set_option_precision(char *argv, LIS_INT opt, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='1' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[opt]);
 #else
 		sscanf(argv, "%d", &solver->options[opt]);
@@ -1408,13 +1444,13 @@ LIS_INT lis_solver_set_option_precision(char *argv, LIS_INT opt, LIS_SOLVER solv
 #define __FUNC__ "lis_solver_set_option_storage"
 LIS_INT lis_solver_set_option_storage(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='9' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_STORAGE]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_STORAGE]);
@@ -1439,13 +1475,13 @@ LIS_INT lis_solver_set_option_storage(char *argv, LIS_SOLVER solver)
 #define __FUNC__ "lis_solver_set_option_conv_cond"
 LIS_INT lis_solver_set_option_conv_cond(char *argv, LIS_SOLVER solver)
 {
-	LIS_INT  i;
+	LIS_INT i;
 
 	LIS_DEBUG_FUNC_IN;
 
 	if( argv[0]>='0' && argv[0]<='3' )
 	{
-#ifdef _LONGLONG
+#ifdef _LONG__LONG
 		sscanf(argv, "%lld", &solver->options[LIS_OPTIONS_CONV_COND]);
 #else
 		sscanf(argv, "%d", &solver->options[LIS_OPTIONS_CONV_COND]);
@@ -1467,26 +1503,26 @@ LIS_INT lis_solver_set_option_conv_cond(char *argv, LIS_SOLVER solver)
 }
 
 #undef __FUNC__
-#define __FUNC__ "lis_solver_get_iters"
-LIS_INT lis_solver_get_iters(LIS_SOLVER solver, LIS_INT *iters)
+#define __FUNC__ "lis_solver_get_iter"
+LIS_INT lis_solver_get_iter(LIS_SOLVER solver, LIS_INT *iter)
 {
 	LIS_DEBUG_FUNC_IN;
 
-	*iters = solver->iter;
+	*iter = solver->iter;
 
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 }
 
 #undef __FUNC__
-#define __FUNC__ "lis_solver_get_itersex"
-LIS_INT lis_solver_get_itersex(LIS_SOLVER solver, LIS_INT *iters, LIS_INT *iters_double, LIS_INT *iters_quad)
+#define __FUNC__ "lis_solver_get_iterex"
+LIS_INT lis_solver_get_iterex(LIS_SOLVER solver, LIS_INT *iter, LIS_INT *iter_double, LIS_INT *iter_quad)
 {
 	LIS_DEBUG_FUNC_IN;
 
-	*iters = solver->iter;
-	*iters_double = solver->iter2;
-	*iters_quad = solver->iter - solver->iter2;
+	*iter = solver->iter;
+	*iter_double = solver->iter2;
+	*iter_quad = solver->iter - solver->iter2;
 
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
@@ -1494,11 +1530,11 @@ LIS_INT lis_solver_get_itersex(LIS_SOLVER solver, LIS_INT *iters, LIS_INT *iters
 
 #undef __FUNC__
 #define __FUNC__ "lis_solver_get_time"
-LIS_INT lis_solver_get_time(LIS_SOLVER solver, double *times)
+LIS_INT lis_solver_get_time(LIS_SOLVER solver, double *time)
 {
 	LIS_DEBUG_FUNC_IN;
 
-	*times  = solver->times;
+	*time  = solver->time;
 
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
@@ -1506,15 +1542,15 @@ LIS_INT lis_solver_get_time(LIS_SOLVER solver, double *times)
 
 #undef __FUNC__
 #define __FUNC__ "lis_solver_get_timeex"
-LIS_INT lis_solver_get_timeex(LIS_SOLVER solver, double *times, double *itimes, double *ptimes, double *p_c_times, double *p_i_times)
+LIS_INT lis_solver_get_timeex(LIS_SOLVER solver, double *time, double *itime, double *ptime, double *p_c_time, double *p_i_time)
 {
 	LIS_DEBUG_FUNC_IN;
 
-	*times  = solver->times;
-	if( itimes ) *itimes = solver->itimes;
-	if( ptimes ) *ptimes = solver->ptimes;
-	if( p_c_times ) *p_c_times = solver->p_c_times;
-	if( p_i_times ) *p_i_times = solver->p_i_times;
+	*time  = solver->time;
+	if( itime ) *itime = solver->itime;
+	if( ptime ) *ptime = solver->ptime;
+	if( p_c_time ) *p_c_time = solver->p_c_time;
+	if( p_i_time ) *p_i_time = solver->p_i_time;
 
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
@@ -1572,7 +1608,7 @@ LIS_INT lis_solver_get_status(LIS_SOLVER solver, LIS_INT *status)
 #define __FUNC__ "lis_solver_get_rhistory"
 LIS_INT lis_solver_get_rhistory(LIS_SOLVER solver, LIS_VECTOR v)
 {
-	LIS_INT		i,n,maxiter;
+	LIS_INT	i,n,maxiter;
 
 	maxiter = solver->iter+1;
         if( solver->retcode!=LIS_SUCCESS )
@@ -1582,7 +1618,7 @@ LIS_INT lis_solver_get_rhistory(LIS_SOLVER solver, LIS_VECTOR v)
 	n = _min(v->n,maxiter);
 	for(i=0;i<n;i++)
 	{
-		v->value[i] = solver->residual[i];
+		v->value[i] = solver->rhistory[i];
 	}
 	return LIS_SUCCESS;
 }
@@ -1650,7 +1686,7 @@ LIS_INT lis_solver_get_residual_nrm1_b(LIS_VECTOR r, LIS_SOLVER solver, LIS_REAL
 #define __FUNC__ "lis_solver_set_shadowresidual"
 LIS_INT lis_solver_set_shadowresidual(LIS_SOLVER solver, LIS_VECTOR r0, LIS_VECTOR rs0)
 {
-    unsigned long init[4]={0x123, 0x234, 0x345, 0x456}, length=4;
+        unsigned long init[4]={0x123, 0x234, 0x345, 0x456}, length=4;
 	LIS_INT i,n,resid;
 
 	LIS_DEBUG_FUNC_IN;
@@ -1706,4 +1742,3 @@ LIS_INT lis_solver_set_shadowresidual(LIS_SOLVER solver, LIS_VECTOR r0, LIS_VECT
 	LIS_DEBUG_FUNC_OUT;
 	return LIS_SUCCESS;
 }
-
