@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -76,7 +76,7 @@
    r = s - omega*t
    ***************************************************/
 
-#define NWORK				7
+#define NWORK 7
 #undef __FUNC__
 #define __FUNC__ "lis_bicgstab_check_params"
 LIS_INT lis_bicgstab_check_params(LIS_SOLVER solver)
@@ -90,8 +90,8 @@ LIS_INT lis_bicgstab_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_bicgstab_malloc_work"
 LIS_INT lis_bicgstab_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -137,25 +137,21 @@ LIS_INT lis_bicgstab_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_bicgstab(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,rtld, t,p,v, s, phat, shat;
 	LIS_SCALAR alpha, beta, omega, rho, rho_old, tmpdot1, tmpdot2;
 	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -215,12 +211,12 @@ LIS_INT lis_bicgstab(LIS_SOLVER solver)
 		}
 		
 		/* phat = M^-1 * p */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, p, phat);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 
 		/* v = A * phat */
-		LIS_MATVEC(A,phat,v);
+		lis_matvec(A,phat,v);
 
 		/* tmpdot1 = <rtld,v> */
 		lis_vector_dot(rtld,v,&tmpdot1);
@@ -241,7 +237,7 @@ LIS_INT lis_bicgstab(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -249,18 +245,18 @@ LIS_INT lis_bicgstab(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/* shat = M^-1 * s */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, s, shat);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 
 		/* t = A * shat */
-		LIS_MATVEC(A,shat,t);
+		lis_matvec(A,shat,t);
 
 		/* tmpdot1 = <t,s> */
 		/* tmpdot2 = <t,t> */
@@ -283,7 +279,7 @@ LIS_INT lis_bicgstab(LIS_SOLVER solver)
 
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -292,7 +288,7 @@ LIS_INT lis_bicgstab(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -321,25 +317,21 @@ LIS_INT lis_bicgstab(LIS_SOLVER solver)
 LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,rtld, t,p,v, s, phat, shat;
 	LIS_QUAD_PTR alpha, beta, omega, rho, rho_old, tmpdot1, tmpdot2;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -413,12 +405,12 @@ LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 		}
 		
 		/* phat = M^-1 * p */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, p, phat);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 
 		/* v = A * phat */
-		LIS_MATVEC(A,phat,v);
+		lis_matvec(A,phat,v);
 
 		/* tmpdot1 = <rtld,v> */
 		lis_vector_dotex_mmm(rtld,v,&tmpdot1);
@@ -438,7 +430,7 @@ LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -447,18 +439,18 @@ LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/* shat = M^-1 * s */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, s, shat);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 
 		/* t = A * shat */
-		LIS_MATVEC(A,shat,t);
+		lis_matvec(A,shat,t);
 
 		/* tmpdot1 = <t,s> */
 		/* tmpdot2 = <t,t> */
@@ -481,7 +473,7 @@ LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -490,7 +482,7 @@ LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -520,29 +512,25 @@ LIS_INT lis_bicgstab_quad(LIS_SOLVER solver)
 LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,rtld, t,p,v, s, phat, shat;
 	LIS_QUAD_PTR alpha, beta, omega, rho, rho_old, tmpdot1, tmpdot2;
-	LIS_REAL   bnrm2, nrm2, tol, tol2;
-	LIS_INT iter,maxiter,n,output,conv;
+	LIS_REAL bnrm2, nrm2, tol, tol2;
+	LIS_INT iter,maxiter,output,conv;
 	LIS_INT iter2,maxiter2;
-	double times,ptimes;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter  = solver->options[LIS_OPTIONS_MAXITER];
 	maxiter2 = solver->options[LIS_OPTIONS_SWITCH_MAXITER];
 	output   = solver->options[LIS_OPTIONS_OUTPUT];
-	conv    = solver->options[LIS_OPTIONS_CONV_COND];
+	conv     = solver->options[LIS_OPTIONS_CONV_COND];
 	tol      = solver->params[LIS_PARAMS_RESID-LIS_OPTIONS_LEN];
 	tol2     = solver->params[LIS_PARAMS_SWITCH_RESID-LIS_OPTIONS_LEN];
-	ptimes  = 0.0;
+	ptime    = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -617,12 +605,12 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 			}
 			
 			/* phat = M^-1 * p */
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, p, phat);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 
 			/* v = A * phat */
-			LIS_MATVEC(A,phat,v);
+			lis_matvec(A,phat,v);
 
 			/* tmpdot1 = <rtld,v> */
 			lis_vector_dot(rtld,v,&tmpdot1.hi[0]);
@@ -641,24 +629,24 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 			{
 				if( output )
 				{
-					if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+					if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 					if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 				}
 
 				lis_vector_axpy(alpha.hi[0],phat,x);
 				solver->iter       = iter;
-				solver->iter2     = iter;
-				solver->ptimes     = ptimes;
+				solver->iter2      = iter;
+				solver->ptime      = ptime;
 				break;
 			}
 
 			/* shat = M^-1 * s */
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, s, shat);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 
 			/* t = A * shat */
-			LIS_MATVEC(A,shat,t);
+			lis_matvec(A,shat,t);
 
 			/* tmpdot1 = <t,s> */
 			/* tmpdot2 = <t,t> */
@@ -678,15 +666,15 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 			lis_solver_get_residual[conv](r,solver,&nrm2);
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
 			if( nrm2 <= tol2 )
 			{
 				solver->iter       = iter;
-				solver->iter2     = iter;
-				solver->ptimes     = ptimes;
+				solver->iter2      = iter;
+				solver->ptime      = ptime;
 				break;
 			}
 			
@@ -758,12 +746,12 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 			}
 			
 			/* phat = M^-1 * p */
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, p, phat);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 
 			/* v = A * phat */
-			LIS_MATVEC(A,phat,v);
+			lis_matvec(A,phat,v);
 
 			/* tmpdot1 = <rtld,v> */
 			lis_vector_dotex_mmm(rtld,v,&tmpdot1);
@@ -783,7 +771,7 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 			{
 				if( output )
 				{
-					if( output & LIS_PRINT_MEM ) solver->residual[iter2] = nrm2;
+					if( output & LIS_PRINT_MEM ) solver->rhistory[iter2] = nrm2;
 					if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 				}
 
@@ -793,18 +781,18 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 				solver->iter       = iter2;
 				solver->iter2      = iter;
 				solver->resid      = nrm2;
-				solver->ptimes     = ptimes;
+				solver->ptime      = ptime;
 				LIS_DEBUG_FUNC_OUT;
 				return LIS_SUCCESS;
 			}
 
 			/* shat = M^-1 * s */
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, s, shat);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 
 			/* t = A * shat */
-			LIS_MATVEC(A,shat,t);
+			lis_matvec(A,shat,t);
 
 			/* tmpdot1 = <t,s> */
 			/* tmpdot2 = <t,t> */
@@ -827,7 +815,7 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 			lis_solver_get_residual[conv](r,solver,&nrm2);
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter2] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter2] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -837,7 +825,7 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
 				solver->iter       = iter2;
 				solver->iter2      = iter;
 				solver->resid      = nrm2;
-				solver->ptimes     = ptimes;
+				solver->ptime      = ptime;
 				LIS_DEBUG_FUNC_OUT;
 				return LIS_SUCCESS;
 			}
@@ -893,7 +881,7 @@ LIS_INT lis_bicgstab_switch(LIS_SOLVER solver)
    p(k)     = z(k) + beta*(p(k-1) - omega*map(k-1))
    ***************************************************/
 #undef NWORK
-#define NWORK				9
+#define NWORK 9
 #undef __FUNC__
 #define __FUNC__ "lis_bicrstab_check_params"
 LIS_INT lis_bicrstab_check_params(LIS_SOLVER solver)
@@ -907,8 +895,8 @@ LIS_INT lis_bicrstab_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_bicrstab_malloc_work"
 LIS_INT lis_bicrstab_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -954,25 +942,21 @@ LIS_INT lis_bicrstab_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_bicrstab(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,rtld, p, s, ap, ms, map, ams, z;
 	LIS_SCALAR alpha, beta, omega, rho, rho_old, tmpdot1, tmpdot2;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -994,10 +978,10 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,p);
 
-	LIS_MATVECT(A,p,rtld);
-	times = lis_wtime();
+	lis_matvect(A,p,rtld);
+	time = lis_wtime();
 	lis_psolve(solver, r, z);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_copy(z,p);
 	lis_vector_dot(rtld,z,&rho_old);
 	
@@ -1008,10 +992,10 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 		/* tmpdot1 = <rtld,map>        */
 		/* alpha   = rho_old / tmpdot1 */
 		/* s       = r - alpha*ap      */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,map,&tmpdot1);
 		alpha = rho_old / tmpdot1;
 		lis_vector_axpyz(-alpha,ap,r,s);
@@ -1022,7 +1006,7 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -1030,7 +1014,7 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -1041,7 +1025,7 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 		/* tmpdot2 = <ams,ams>         */
 		/* omega   = tmpdot1 / tmpdot2 */
 		lis_vector_axpyz(-alpha,map,z,ms);
-		LIS_MATVEC(A,ms,ams);
+		lis_matvec(A,ms,ams);
 		lis_vector_dot(ams,s,&tmpdot1);
 		lis_vector_dot(ams,ams,&tmpdot2);
 		omega   = tmpdot1 / tmpdot2;
@@ -1056,7 +1040,7 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -1065,16 +1049,16 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 		
 		/* z   = M^-1 * r */
 		/* rho = <rtld,z> */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, z);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,z,&rho);
 		if( rho==0.0 )
 		{
@@ -1107,25 +1091,21 @@ LIS_INT lis_bicrstab(LIS_SOLVER solver)
 LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,rtld, p, s, ap, ms, map, ams, z;
 	LIS_QUAD_PTR alpha, beta, omega, rho, rho_old, tmpdot1, tmpdot2;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -1154,10 +1134,10 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,p);
 
-	LIS_MATVECT(A,p,rtld);
-	times = lis_wtime();
+	lis_matvect(A,p,rtld);
+	time = lis_wtime();
 	lis_psolve(solver, r, z);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_copyex_mm(z,p);
 	lis_vector_dotex_mmm(rtld,z,&rho_old);
 	
@@ -1168,10 +1148,10 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 		/* tmpdot1 = <rtld,map>        */
 		/* alpha   = rho_old / tmpdot1 */
 		/* s       = r - alpha*ap      */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,map,&tmpdot1);
 		lis_quad_div((LIS_QUAD *)alpha.hi,(LIS_QUAD *)rho_old.hi,(LIS_QUAD *)tmpdot1.hi);
 		lis_quad_minus((LIS_QUAD *)alpha.hi);
@@ -1183,7 +1163,7 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -1192,7 +1172,7 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -1203,7 +1183,7 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 		/* tmpdot2 = <ams,ams>         */
 		/* omega   = tmpdot1 / tmpdot2 */
 		lis_vector_axpyzex_mmmm(alpha,map,z,ms);
-		LIS_MATVEC(A,ms,ams);
+		lis_matvec(A,ms,ams);
 		lis_vector_dotex_mmm(ams,s,&tmpdot1);
 		lis_vector_dotex_mmm(ams,ams,&tmpdot2);
 		lis_quad_div((LIS_QUAD *)omega.hi,(LIS_QUAD *)tmpdot1.hi,(LIS_QUAD *)tmpdot2.hi);
@@ -1220,7 +1200,7 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -1229,16 +1209,16 @@ LIS_INT lis_bicrstab_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 		
 		/* z   = M^-1 * r */
 		/* rho = <rtld,z> */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, z);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,z,&rho);
 		if( rho.hi[0]==0.0 && rho.lo[0]==0.0 )
 		{
