@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -47,7 +47,7 @@
 #endif
 #include "lislib.h"
 
-#define NWORK				4
+#define NWORK 4
 /************************************************
  * lis_bicgstabl_check_params
  * lis_bicgstabl_malloc_work
@@ -75,8 +75,8 @@ LIS_INT lis_bicgstabl_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_bicgstabl_malloc_work"
 LIS_INT lis_bicgstabl_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,l,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,l,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -123,40 +123,31 @@ LIS_INT lis_bicgstabl_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR rtld, bp,t,xp, *r,*u;
-	LIS_SCALAR *tau, *gamma, *gamma1, *gamma2, *sigma;
+	LIS_SCALAR *tau, *gamma, *gamma1, *gamma2;
+	LIS_SCALAR *sigma;
 	LIS_SCALAR alpha, beta, omega, rho0, rho1;
-	LIS_SCALAR rnorm0, rnorm;
-	LIS_SCALAR normx, normr;
-	LIS_SCALAR delta, one, zero, onem;
+	LIS_REAL rnorm0, rnorm;
+	LIS_REAL normx, normr;
 	LIS_SCALAR nu;
 
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_INT l,i,j;
-	LIS_INT w_dim, z_dim;
-	LIS_INT rcmp, xpdt;
-	LIS_INT ione, izero;
-
-
+	LIS_INT z_dim;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	l       = solver->options[LIS_OPTIONS_ELL];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
-	w_dim   = n;
+	ptime   = 0.0;
 	z_dim   = l+1;
 
 	rtld    = solver->work[0];
@@ -177,13 +168,6 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 	gamma2  = &gamma1[z_dim];
 	sigma   = &gamma2[z_dim];
 
-	one     = (LIS_SCALAR)1.0;
-	onem    = (LIS_SCALAR)-1.0;
-	zero    = (LIS_SCALAR)0.0;
-	ione    = 1;
-	izero   = 0;
-
-
 	/* Initial Residual */
 	if( lis_solver_get_initial_residual(solver,NULL,NULL,r[0],&bnrm2) )
 	{
@@ -202,14 +186,10 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 	rnorm  = rnorm0;
 	normx  = rnorm0;
 	normr  = rnorm0;
-	rcmp   = 0;
-	xpdt   = 0;
 
 	alpha = (LIS_SCALAR)0.0;
 	omega = (LIS_SCALAR)1.0;
 	rho0  = (LIS_SCALAR)1.0;
-	delta = (LIS_SCALAR)1.0e-2;
-
 
 	iter = 0;
 	while( iter<=maxiter )
@@ -230,10 +210,10 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 			/* test breakdown */
 			if( rho1==0.0 )
 			{
-				times = lis_wtime();
+				time = lis_wtime();
 				lis_psolve(solver, x, t);
 				lis_vector_copy(t, x);
-				ptimes += lis_wtime()-times;
+				ptime += lis_wtime()-time;
 				lis_vector_axpy(1.0,xp,x);
 				solver->retcode   = LIS_BREAKDOWN;
 				solver->iter      = iter;
@@ -256,10 +236,10 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 
 			/* u[j+1] = A    * u[j]   */
 			/* u[j+1] = M^-1 * u[j+1] */
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, u[j], t);
-			ptimes += lis_wtime()-times;
-			LIS_MATVEC(A,t,u[j+1]);
+			ptime += lis_wtime()-time;
+			lis_matvec(A,t,u[j+1]);
 
 			/* nu = <rtld, u[j+1]> */
 			lis_vector_dot(rtld,u[j+1],&nu);
@@ -267,10 +247,10 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 			/* test breakdown */
 			if( nu==0.0 )
 			{
-				times = lis_wtime();
+				time = lis_wtime();
 				lis_psolve(solver, x, t);
 				lis_vector_copy(t, x);
-				ptimes += lis_wtime()-times;
+				ptime += lis_wtime()-time;
 				lis_vector_axpy(1.0,xp,x);
 				solver->retcode   = LIS_BREAKDOWN;
 				solver->iter      = iter;
@@ -298,7 +278,7 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 			{
 				if( output )
 				{
-					if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+					if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 					if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 				}
 			}
@@ -307,18 +287,18 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 			{
 				if( output )
 				{
-					if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+					if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 					if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 				}	
-				times = lis_wtime();
+				time = lis_wtime();
 				lis_psolve(solver, x, t);
 				lis_vector_copy(t, x);
-				ptimes += lis_wtime()-times;
+				ptime += lis_wtime()-time;
 				lis_vector_axpy(1.0,xp,x);
 				solver->retcode    = LIS_SUCCESS;
 				solver->iter       = iter;
 				solver->resid      = nrm2;
-				solver->ptimes     = ptimes;
+				solver->ptime     = ptime;
 				lis_free(tau);
 				LIS_DEBUG_FUNC_OUT;
 				return LIS_SUCCESS;
@@ -326,10 +306,10 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 
 			/* r[j+1] = A    * r[j]   */
 			/* r[j+1] = M^-1 * r[j+1] */
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, r[j], t);
-			ptimes += lis_wtime()-times;
-			LIS_MATVEC(A,t,r[j+1]);
+			ptime += lis_wtime()-time;
+			lis_matvec(A,t,r[j+1]);
 
 			lis_vector_nrm2(r[0],&rnorm);
 			normx = _max(normx,rnorm);
@@ -390,21 +370,21 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r[0],solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
 		if( tol >= nrm2 )
 		{
-			times = lis_wtime();
+			time = lis_wtime();
 			lis_psolve(solver, x, t);
 			lis_vector_copy(t, x);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 			lis_vector_axpy(1.0,xp,x);
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			lis_free(tau);
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
@@ -425,8 +405,7 @@ LIS_INT lis_bicgstabl(LIS_SOLVER solver)
 LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR rtld, bp,t,xp, *r,*u;
 	LIS_QUAD *tau, *gamma, *gamma1, *gamma2, *sigma;
 	LIS_QUAD_PTR alpha, beta, omega, rho0, rho1,sigtmp;
@@ -435,29 +414,24 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 	LIS_QUAD_PTR delta, one, zero, onem;
 	LIS_QUAD_PTR nu;
 
-	LIS_REAL   bnrm2, nrm2, tol;
+	LIS_REAL bnrm2, nrm2, tol;
 	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	double time,ptime;
 
 	LIS_INT l,i,j;
-	LIS_INT w_dim, z_dim;
-	LIS_INT rcmp, xpdt;
-	LIS_INT ione, izero;
+	LIS_INT z_dim;
 
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
 	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	l       = solver->options[LIS_OPTIONS_ELL];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
-	w_dim   = n;
+	ptime   = 0.0;
 	z_dim   = l+1;
 
 	rtld    = solver->work[0];
@@ -500,8 +474,6 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 	onem.lo[0]    = 0.0;
 	zero.hi[0]    = 0.0;
 	zero.lo[0]    = 0.0;
-	ione    = 1;
-	izero   = 0;
 
 
 	/* Initial Residual */
@@ -513,9 +485,9 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 	}
 	tol     = solver->tol;
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver,t,r[0]);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_solver_set_shadowresidual(solver,r[0],rtld);
 
 	lis_vector_copyex_mm(r[0],bp);
@@ -528,8 +500,6 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 	normx.lo[0]  = rnorm0.lo[0];
 	normr.hi[0]  = rnorm0.hi[0];
 	normr.lo[0]  = rnorm0.lo[0];
-	rcmp   = 0;
-	xpdt   = 0;
 
 	alpha.hi[0] = 0.0;
 	alpha.lo[0] = 0.0;
@@ -585,10 +555,10 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 
 			/* u[j+1] = A    * u[j]   */
 			/* u[j+1] = M^-1 * u[j+1] */
-			LIS_MATVEC(A,u[j],t);
-			times = lis_wtime();
+			lis_matvec(A,u[j],t);
+			time = lis_wtime();
 			lis_psolve(solver, t, u[j+1]);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 
 			/* nu = <rtld, u[j+1]> */
 			lis_vector_dotex_mmm(rtld,u[j+1],&nu);
@@ -624,7 +594,7 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 			{
 				if( output )
 				{
-					if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+					if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 					if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 				}
 			}
@@ -633,14 +603,14 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 			{
 				if( output )
 				{
-					if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+					if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 					if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 				}	
 				lis_vector_axpyex_mmm(one,xp,x);
 				solver->retcode    = LIS_SUCCESS;
 				solver->iter       = iter;
 				solver->resid      = nrm2;
-				solver->ptimes     = ptimes;
+				solver->ptime      = ptime;
 				lis_free(tau);
 				LIS_DEBUG_FUNC_OUT;
 				return LIS_SUCCESS;
@@ -648,10 +618,10 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 
 			/* r[j+1] = A    * r[j]   */
 			/* r[j+1] = M^-1 * r[j+1] */
-			LIS_MATVEC(A,r[j],t);
-			times = lis_wtime();
+			lis_matvec(A,r[j],t);
+			time = lis_wtime();
 			lis_psolve(solver, t, r[j+1]);
-			ptimes += lis_wtime()-times;
+			ptime += lis_wtime()-time;
 
 			lis_vector_nrm2ex_mm(r[0],&rnorm);
 			lis_quad_max((LIS_QUAD *)normx.hi,(LIS_QUAD *)rnorm.hi,(LIS_QUAD *)normx.hi);
@@ -741,7 +711,7 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r[0],solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -751,7 +721,7 @@ LIS_INT lis_bicgstabl_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			lis_free(tau);
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
