@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -69,7 +69,7 @@
    r(k)      = r(k-1) - alpha*q(k)
  ***************************************/
 
-#define NWORK				4
+#define NWORK 4
 #undef __FUNC__
 #define __FUNC__ "lis_cg_check_params"
 LIS_INT lis_cg_check_params(LIS_SOLVER solver)
@@ -83,8 +83,8 @@ LIS_INT lis_cg_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_cg_malloc_work"
 LIS_INT lis_cg_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -129,26 +129,22 @@ LIS_INT lis_cg_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_cg(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,z,p,q;
 	LIS_SCALAR alpha, beta, rho, rho_old;
 	LIS_SCALAR dot_pq;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 
 	z       = solver->work[0];
@@ -171,9 +167,9 @@ LIS_INT lis_cg(LIS_SOLVER solver)
 	for( iter=1; iter<=maxiter; iter++ )
 	{
 		/* z = M^-1 * r */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver,r,z);
-		ptimes += lis_wtime() - times;
+		ptime += lis_wtime() - time;
 
 		/* rho = <r,z> */
 		lis_vector_dot(r,z,&rho);
@@ -185,7 +181,7 @@ LIS_INT lis_cg(LIS_SOLVER solver)
 		lis_vector_xpay(z,beta,p);
 		
 		/* q = Ap */
-		LIS_MATVEC(A,p,q);
+		lis_matvec(A,p,q);
 		
 		/* dot_pq = <p,q> */
 		lis_vector_dot(p,q,&dot_pq);
@@ -213,12 +209,8 @@ LIS_INT lis_cg(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
-#ifdef _LONGLONG
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5lld  residual = %e\n", iter, nrm2);
-#else
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", iter, nrm2);
-#endif
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
+			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 		
 		if( tol >= nrm2 )
@@ -226,7 +218,7 @@ LIS_INT lis_cg(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -246,26 +238,22 @@ LIS_INT lis_cg(LIS_SOLVER solver)
 LIS_INT lis_cg_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,z,p,q;
 	LIS_QUAD_PTR alpha, beta, rho, rho_old;
 	LIS_QUAD_PTR dot_pq;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	LIS_QUAD_SCALAR_MALLOC(alpha,0,1);
 	LIS_QUAD_SCALAR_MALLOC(beta,1,1);
@@ -294,9 +282,9 @@ LIS_INT lis_cg_quad(LIS_SOLVER solver)
 	for( iter=1; iter<=maxiter; iter++ )
 	{
 		/* z = M^-1 * r */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver,r,z);
-		ptimes += lis_wtime() - times;
+		ptime += lis_wtime() - time;
 
 		/* rho = <r,z> */
 		lis_vector_dotex_mmm(r,z,&rho);
@@ -308,7 +296,7 @@ LIS_INT lis_cg_quad(LIS_SOLVER solver)
 		lis_vector_xpayex_mmm(z,beta,p);
 
 		/* q = Ap */
-		LIS_MATVEC(A,p,q);
+		lis_matvec(A,p,q);
 		
 		/* dot_pq = <p,q> */
 		lis_vector_dotex_mmm(p,q,&dot_pq);
@@ -337,12 +325,8 @@ LIS_INT lis_cg_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
-#ifdef _LONGLONG
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5lld  residual = %e\n", iter, nrm2);
-#else
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", iter, nrm2);
-#endif
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
+			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 		
 		if( tol > nrm2 )
@@ -350,7 +334,7 @@ LIS_INT lis_cg_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -370,30 +354,26 @@ LIS_INT lis_cg_quad(LIS_SOLVER solver)
 LIS_INT lis_cg_switch(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,z,p,q;
 	LIS_QUAD_PTR alpha, beta, rho, rho_old;
 	LIS_QUAD_PTR dot_pq;
-	LIS_REAL   bnrm2, nrm2, tol, tol2;
-	LIS_INT iter,maxiter,n,output,conv;
+	LIS_REAL bnrm2, nrm2, tol, tol2;
+	LIS_INT iter,maxiter,output,conv;
 	LIS_INT iter2,maxiter2;
-	double times,ptimes;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter  = solver->options[LIS_OPTIONS_MAXITER];
 	maxiter2 = solver->options[LIS_OPTIONS_SWITCH_MAXITER];
 	output   = solver->options[LIS_OPTIONS_OUTPUT];
-	conv    = solver->options[LIS_OPTIONS_CONV_COND];
+	conv     = solver->options[LIS_OPTIONS_CONV_COND];
 	tol      = solver->params[LIS_PARAMS_RESID-LIS_OPTIONS_LEN];
 	tol2     = solver->params[LIS_PARAMS_SWITCH_RESID-LIS_OPTIONS_LEN];
-	ptimes  = 0.0;
+	ptime    = 0.0;
 
 	LIS_QUAD_SCALAR_MALLOC(alpha,0,1);
 	LIS_QUAD_SCALAR_MALLOC(beta,1,1);
@@ -424,9 +404,9 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 	for( iter=1; iter<=maxiter2; iter++ )
 	{
 		/* z = M^-1 * r */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver,r,z);
-		ptimes += lis_wtime() - times;
+		ptime += lis_wtime() - time;
 
 		/* rho = <r,z> */
 		lis_vector_dot(r,z,&rho.hi[0]);
@@ -438,7 +418,7 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 		lis_vector_xpay(z,beta.hi[0],p);
 		
 		/* q = Ap */
-		LIS_MATVEC(A,p,q);
+		lis_matvec(A,p,q);
 		
 		/* dot_pq = <p,q> */
 		lis_vector_dot(p,q,&dot_pq.hi[0]);
@@ -467,19 +447,15 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
-#ifdef _LONGLONG
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5lld  residual = %e\n", iter, nrm2);
-#else
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", iter, nrm2);
-#endif
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
+			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 		
 		if( tol2 >= nrm2 )
 		{
 			solver->iter       = iter;
-			solver->iter2     = iter;
-			solver->ptimes     = ptimes;
+			solver->iter2      = iter;
+			solver->ptime      = ptime;
 			break;
 		}
 		rho_old.hi[0] = rho.hi[0];
@@ -500,9 +476,9 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 	for( iter2=iter+1; iter2<=maxiter; iter2++ )
 	{
 		/* z = M^-1 * r */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver,r,z);
-		ptimes += lis_wtime() - times;
+		ptime += lis_wtime() - time;
 
 		/* rho = <r,z> */
 		lis_vector_dotex_mmm(r,z,&rho);
@@ -514,7 +490,7 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 		lis_vector_xpayex_mmm(z,beta,p);
 
 		/* q = Ap */
-		LIS_MATVEC(A,p,q);
+		lis_matvec(A,p,q);
 		
 		/* dot_pq = <p,q> */
 		lis_vector_dotex_mmm(p,q,&dot_pq);
@@ -543,12 +519,8 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter2] = nrm2;
-#ifdef _LONGLONG
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5lld  residual = %e\n", iter, nrm2);
-#else
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", iter, nrm2);
-#endif
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter2] = nrm2;
+			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 		
 		if( tol > nrm2 )
@@ -556,7 +528,7 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter2;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -595,7 +567,7 @@ LIS_INT lis_cg_switch(LIS_SOLVER solver)
    q(k)      = az(k) + beta*q(k-1)
  ***************************************/
 #undef NWORK
-#define NWORK				6
+#define NWORK 6
 #undef __FUNC__
 #define __FUNC__ "lis_cr_check_params"
 LIS_INT lis_cr_check_params(LIS_SOLVER solver)
@@ -609,8 +581,8 @@ LIS_INT lis_cr_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_cr_malloc_work"
 LIS_INT lis_cr_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -655,26 +627,22 @@ LIS_INT lis_cr_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_cr(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,z,p,q, qtld, az;
 	LIS_SCALAR alpha, beta, rho;
 	LIS_SCALAR dot_rq, dot_zq;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 
 	z       = solver->work[0];
@@ -692,18 +660,18 @@ LIS_INT lis_cr(LIS_SOLVER solver)
 	}
 	tol     = solver->tol;
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver,r,p);
-	ptimes += lis_wtime() - times;
-	LIS_MATVEC(A,p,q);
+	ptime += lis_wtime() - time;
+	lis_matvec(A,p,q);
 	lis_vector_copy(p,z);
 
 	for( iter=1; iter<=maxiter; iter++ )
 	{
 		/* qtld = M^-1 * q */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver,q,qtld);
-		ptimes += lis_wtime() - times;
+		ptime += lis_wtime() - time;
 
 		/* rho = <qtld,q> */
 		lis_vector_dot(qtld,q,&rho);
@@ -734,12 +702,8 @@ LIS_INT lis_cr(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
-#ifdef _LONGLONG
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5lld  residual = %e\n", iter, nrm2);
-#else
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", iter, nrm2);
-#endif
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
+			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 		
 		if( tol >= nrm2 )
@@ -747,7 +711,7 @@ LIS_INT lis_cr(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -756,7 +720,7 @@ LIS_INT lis_cr(LIS_SOLVER solver)
 		lis_vector_axpy(-alpha,qtld,z);
 
 		/* az = Az */
-		LIS_MATVEC(A,z,az);
+		lis_matvec(A,z,az);
 
 		/* dot_zq = <az,qtld> */
 		lis_vector_dot(az,qtld,&dot_zq);
@@ -784,26 +748,22 @@ LIS_INT lis_cr(LIS_SOLVER solver)
 LIS_INT lis_cr_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r,z,p,q, qtld, az;
 	LIS_QUAD_PTR alpha, beta, rho;
 	LIS_QUAD_PTR dot_rq, dot_zq;
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	LIS_QUAD_SCALAR_MALLOC(alpha,0,1);
 	LIS_QUAD_SCALAR_MALLOC(beta,1,1);
@@ -826,18 +786,18 @@ LIS_INT lis_cr_quad(LIS_SOLVER solver)
 	}
 	tol     = solver->tol;
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver,r,p);
-	ptimes += lis_wtime() - times;
-	LIS_MATVEC(A,p,q);
+	ptime += lis_wtime() - time;
+	lis_matvec(A,p,q);
 	lis_vector_copyex_mm(p,z);
 
 	for( iter=1; iter<=maxiter; iter++ )
 	{
 		/* qtld = M^-1 * q */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver,q,qtld);
-		ptimes += lis_wtime() - times;
+		ptime += lis_wtime() - time;
 
 		/* rho = <qtld,q> */
 		lis_vector_dotex_mmm(qtld,q,&rho);
@@ -869,12 +829,8 @@ LIS_INT lis_cr_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
-#ifdef _LONGLONG
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5lld  residual = %e\n", iter, nrm2);
-#else
-			if( output & LIS_PRINT_OUT && A->my_rank==0 ) printf("iter: %5d  residual = %e\n", iter, nrm2);
-#endif
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
+			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 		
 		if( tol >= nrm2 )
@@ -882,7 +838,7 @@ LIS_INT lis_cr_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -891,7 +847,7 @@ LIS_INT lis_cr_quad(LIS_SOLVER solver)
 		lis_vector_axpyex_mmm(alpha,qtld,z);
 
 		/* az = Az */
-		LIS_MATVEC(A,z,az);
+		lis_matvec(A,z,az);
 
 		/* dot_zq = <az,qtld> */
 		lis_vector_dotex_mmm(az,qtld,&dot_zq);
