@@ -1,4 +1,4 @@
-/* Copyright (C) The Scalable Software Infrastructure Project. All rights reserved.
+/* Copyright (C) 2005 The Scalable Software Infrastructure Project. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -85,7 +85,7 @@
    p(k)    = mr(k) + beta*(p(k-1) - u(k-1))
  ********************************************************************/
 
-#define NWORK				14
+#define NWORK 14
 #undef __FUNC__
 #define __FUNC__ "lis_gpbicg_check_params"
 LIS_INT lis_gpbicg_check_params(LIS_SOLVER solver)
@@ -99,8 +99,8 @@ LIS_INT lis_gpbicg_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_gpbicg_malloc_work"
 LIS_INT lis_gpbicg_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -145,27 +145,23 @@ LIS_INT lis_gpbicg_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_gpbicg(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r, rtld, mr, p, ap, map, t, mt_old, mt, amt, y, u, w, z;
 	LIS_SCALAR alpha, beta, rho, rho_old;
 	LIS_SCALAR qsi, eta;
 	LIS_SCALAR tmp, tmpdot[5];
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -194,9 +190,9 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,rtld);
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver, r, p);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_dot(rtld,r,&rho_old);
 	lis_vector_set_all(0,t);
 	lis_vector_set_all(0,w);
@@ -207,10 +203,10 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		/*   ap(k-1) = A * p(k-1)         */
 		/*   map(k-1)= M^-1 * ap(k-1)     */
 		/*   tmpdot0 = <map(k-1),rtld(0)> */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,ap,&tmpdot[0]);
 
 		if( tmpdot[0]==0.0 )
@@ -237,7 +233,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -245,7 +241,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -261,7 +257,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		/*   qsi     = (tmpdot0*tmpdot1-tmpdot2*tmpdot3) / tmp        */
 		/*   eta     = (tmpdot4*tmpdot2-tmpdot3*tmpdot1) / tmp        */
 		lis_vector_axpyz(-alpha,map,mr,mt);
-		LIS_MATVEC(A,mt,amt);
+		lis_matvec(A,mt,amt);
 		lis_vector_dot(y,y,&tmpdot[0]);
 		lis_vector_dot(amt,t,&tmpdot[1]);
 		lis_vector_dot(y,t,&tmpdot[2]);
@@ -302,7 +298,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -311,16 +307,16 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/*   mr(k)   = M^-1 * r(k)                                          */
 		/*   rho(k)  = <rtld,mr(k)>                                         */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, mr);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,r,&rho);
 		if( rho==0.0 )
 		{
@@ -365,9 +361,9 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 	LIS_SCALAR alpha, beta, rho, rho_old;
 	LIS_SCALAR qsi, eta;
 	LIS_SCALAR tmp, tmpdot[5];
-	LIS_REAL   bnrm2, nrm2, tol;
+	LIS_REAL bnrm2, nrm2, tol;
 	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -379,7 +375,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -441,16 +437,16 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		lis_vector_axpyz(beta,ptld,ttld,w);
 
 		/* rhat = M^-1 * r */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, rhat);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 
 		/* p = rhat + beta*(p - u) */
 		lis_vector_axpy(-1,u,p);
 		lis_vector_xpay(rhat,beta,p);
 		
 		/* ptld = A * p */
-		LIS_MATVEC(A,p,ptld);
+		lis_matvec(A,p,ptld);
 
 		/* tmpdot[0] = <rtld,ptld> */
 		lis_vector_dot(rtld,ptld,&tmpdot[0]);
@@ -474,7 +470,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -482,7 +478,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -490,15 +486,15 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		/* that  = M^-1 * t */
 		/* phat  = M^-1 * ptld */
 		/* t0hat = M^-1 * t0 */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, t, that);
 		lis_psolve(solver, ptld, phat);
 		lis_psolve(solver, t0, t0hat);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 
 
 		/* ttld = A * that */
-		LIS_MATVEC(A,that,ttld);
+		lis_matvec(A,that,ttld);
 
 		/* tmpdot[0] = <y,y>       */
 		/* tmpdot[1] = <ttld,t>    */
@@ -545,7 +541,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -554,7 +550,7 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -577,27 +573,23 @@ LIS_INT lis_gpbicg(LIS_SOLVER solver)
 LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r, rtld, mr, p, ap, map, t, mt_old, mt, amt, y, u, w, z;
 	LIS_QUAD_PTR alpha, beta, rho, rho_old;
 	LIS_QUAD_PTR qsi, eta, one;
 	LIS_QUAD_PTR tmp, tmpdot[5];
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -639,9 +631,9 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,rtld);
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver, r, p);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_dotex_mmm(rtld,r,&rho_old);
 	lis_vector_set_allex_nm(0.0,t);
 	lis_vector_set_allex_nm(0.0,w);
@@ -653,10 +645,10 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 		/*   ap(k-1) = A * p(k-1)         */
 		/*   map(k-1)= M^-1 * ap(k-1)     */
 		/*   tmpdot0 = <map(k-1),rtld(0)> */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,ap,&tmpdot[0]);
 
 		if( tmpdot[0].hi[0]==0.0 && tmpdot[0].lo[0] )
@@ -684,7 +676,7 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -693,7 +685,7 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -709,7 +701,7 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 		/*   qsi     = (tmpdot0*tmpdot1-tmpdot2*tmpdot3) / tmp        */
 		/*   eta     = (tmpdot4*tmpdot2-tmpdot3*tmpdot1) / tmp        */
 		lis_vector_axpyzex_mmmm(alpha,map,mr,mt);
-		LIS_MATVEC(A,mt,amt);
+		lis_matvec(A,mt,amt);
 		lis_vector_dotex_mmm(y,y,&tmpdot[0]);
 		lis_vector_dotex_mmm(amt,t,&tmpdot[1]);
 		lis_vector_dotex_mmm(y,t,&tmpdot[2]);
@@ -766,7 +758,7 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -775,16 +767,16 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/*   mr(k)   = M^-1 * r(k)                                          */
 		/*   rho(k)  = <rtld,mr(k)>                                         */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, mr);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,r,&rho);
 		if( rho.hi[0]==0.0 && rho.lo[0]==0.0 )
 		{
@@ -826,31 +818,26 @@ LIS_INT lis_gpbicg_quad(LIS_SOLVER solver)
 LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r, rtld, mr, p, ap, map, t, mt_old, mt, amt, y, u, w, z;
 	LIS_QUAD_PTR alpha, beta, rho, rho_old;
 	LIS_QUAD_PTR qsi, eta, one;
 	LIS_QUAD_PTR tmp, tmpdot[5];
-	LIS_REAL   bnrm2, nrm2, tol, tol2;
-	LIS_INT iter,maxiter,n,output,conv;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
 	LIS_INT iter2,maxiter2;
-	double times,ptimes;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter  = solver->options[LIS_OPTIONS_MAXITER];
 	maxiter2 = solver->options[LIS_OPTIONS_SWITCH_MAXITER];
 	output   = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
 	tol      = solver->params[LIS_PARAMS_RESID-LIS_OPTIONS_LEN];
-	tol2     = solver->params[LIS_PARAMS_SWITCH_RESID-LIS_OPTIONS_LEN];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -899,9 +886,9 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,rtld);
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver, r, p);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_dot(rtld,r,&rho_old.hi[0]);
 	lis_vector_set_all(0,t);
 	lis_vector_set_all(0,w);
@@ -911,10 +898,10 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		/*   ap(k-1) = A * p(k-1)         */
 		/*   map(k-1)= M^-1 * ap(k-1)     */
 		/*   tmpdot0 = <map(k-1),rtld(0)> */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,ap,&tmpdot[0].hi[0]);
 
 		if( tmpdot[0].hi[0]==0.0 )
@@ -942,14 +929,14 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
 			lis_vector_axpy(alpha.hi[0],p,x);
 			solver->iter       = iter;
-			solver->iter2     = iter;
-			solver->ptimes     = ptimes;
+			solver->iter2      = iter;
+			solver->ptime      = ptime;
 			break;
 		}
 
@@ -964,7 +951,7 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		/*   qsi     = (tmpdot0*tmpdot1-tmpdot2*tmpdot3) / tmp        */
 		/*   eta     = (tmpdot4*tmpdot2-tmpdot3*tmpdot1) / tmp        */
 		lis_vector_axpyz(-alpha.hi[0],map,mr,mt);
-		LIS_MATVEC(A,mt,amt);
+		lis_matvec(A,mt,amt);
 		lis_vector_dot(y,y,&tmpdot[0].hi[0]);
 		lis_vector_dot(amt,t,&tmpdot[1].hi[0]);
 		lis_vector_dot(y,t,&tmpdot[2].hi[0]);
@@ -1005,23 +992,23 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
 		if( tol >= nrm2 )
 		{
 			solver->iter       = iter;
-			solver->iter2     = iter;
-			solver->ptimes     = ptimes;
+			solver->iter2      = iter;
+			solver->ptime      = ptime;
 			break;
 		}
 
 		/*   mr(k)   = M^-1 * r(k)                                          */
 		/*   rho(k)  = <rtld,mr(k)>                                         */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, mr);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,r,&rho.hi[0]);
 		if( rho.hi[0]==0.0 )
 		{
@@ -1066,9 +1053,9 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,rtld);
 
-	times = lis_wtime();
+	time = lis_wtime();
 	lis_psolve(solver, r, p);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_dotex_mmm(rtld,r,&rho_old);
 	lis_vector_set_allex_nm(0.0,t);
 	lis_vector_set_allex_nm(0.0,w);
@@ -1078,10 +1065,10 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		/*   ap(k-1) = A * p(k-1)         */
 		/*   map(k-1)= M^-1 * ap(k-1)     */
 		/*   tmpdot0 = <map(k-1),rtld(0)> */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,ap,&tmpdot[0]);
 
 		if( tmpdot[0].hi[0]==0.0 && tmpdot[0].lo[0] )
@@ -1110,7 +1097,7 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter2] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter2] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -1120,7 +1107,7 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 			solver->iter       = iter2;
 			solver->iter2      = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -1136,7 +1123,7 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		/*   qsi     = (tmpdot0*tmpdot1-tmpdot2*tmpdot3) / tmp        */
 		/*   eta     = (tmpdot4*tmpdot2-tmpdot3*tmpdot1) / tmp        */
 		lis_vector_axpyzex_mmmm(alpha,map,mr,mt);
-		LIS_MATVEC(A,mt,amt);
+		lis_matvec(A,mt,amt);
 		lis_vector_dotex_mmm(y,y,&tmpdot[0]);
 		lis_vector_dotex_mmm(amt,t,&tmpdot[1]);
 		lis_vector_dotex_mmm(y,t,&tmpdot[2]);
@@ -1193,7 +1180,7 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter2] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter2] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -1203,16 +1190,16 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
 			solver->iter       = iter2;
 			solver->iter2      = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/*   mr(k)   = M^-1 * r(k)                                          */
 		/*   rho(k)  = <rtld,mr(k)>                                         */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, mr);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,r,&rho);
 		if( rho.hi[0]==0.0 && rho.lo[0]==0.0 )
 		{
@@ -1290,7 +1277,7 @@ LIS_INT lis_gpbicg_switch(LIS_SOLVER solver)
    p(k)    = mr(k) + beta*(p(k-1) - u(k-1))
  ********************************************************************/
 #undef NWORK
-#define NWORK				14
+#define NWORK 14
 #undef __FUNC__
 #define __FUNC__ "lis_gpbicr_check_params"
 LIS_INT lis_gpbicr_check_params(LIS_SOLVER solver)
@@ -1304,8 +1291,8 @@ LIS_INT lis_gpbicr_check_params(LIS_SOLVER solver)
 #define __FUNC__ "lis_gpbicg_malloc_work"
 LIS_INT lis_gpbicr_malloc_work(LIS_SOLVER solver)
 {
-	LIS_VECTOR	*work;
-	LIS_INT			i,j,worklen,err;
+	LIS_VECTOR *work;
+	LIS_INT	i,j,worklen,err;
 
 	LIS_DEBUG_FUNC_IN;
 
@@ -1350,27 +1337,23 @@ LIS_INT lis_gpbicr_malloc_work(LIS_SOLVER solver)
 LIS_INT lis_gpbicr(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r, rtld, mr, p, ap, map, t, mt_old, mt, amt, y, u, w, z;
 	LIS_SCALAR alpha, beta, rho, rho_old;
 	LIS_SCALAR qsi, eta;
 	LIS_SCALAR tmp, tmpdot[5];
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -1399,10 +1382,10 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,p);
 
-	LIS_MATVECT(A,p,rtld);
-	times = lis_wtime();
+	lis_matvect(A,p,rtld);
+	time = lis_wtime();
 	lis_psolve(solver, r, p);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_dot(rtld,p,&rho_old);
 	lis_vector_set_all(0,t);
 	lis_vector_set_all(0,w);
@@ -1413,10 +1396,10 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 		/*   ap(k-1) = A * p(k-1)         */
 		/*   map(k-1)= M^-1 * ap(k-1)     */
 		/*   tmpdot0 = <map(k-1),rtld(0)> */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,map,&tmpdot[0]);
 
 		if( tmpdot[0]==0.0 )
@@ -1443,7 +1426,7 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -1451,7 +1434,7 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -1467,7 +1450,7 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 		/*   qsi     = (tmpdot0*tmpdot1-tmpdot2*tmpdot3) / tmp        */
 		/*   eta     = (tmpdot4*tmpdot2-tmpdot3*tmpdot1) / tmp        */
 		lis_vector_axpyz(-alpha,map,mr,mt);
-		LIS_MATVEC(A,mt,amt);
+		lis_matvec(A,mt,amt);
 		lis_vector_dot(y,y,&tmpdot[0]);
 		lis_vector_dot(amt,t,&tmpdot[1]);
 		lis_vector_dot(y,t,&tmpdot[2]);
@@ -1508,7 +1491,7 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -1517,16 +1500,16 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/*   mr(k)   = M^-1 * r(k)                                          */
 		/*   rho(k)  = <rtld,mr(k)>                                         */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, mr);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dot(rtld,mr,&rho);
 		if( rho==0.0 )
 		{
@@ -1563,27 +1546,23 @@ LIS_INT lis_gpbicr(LIS_SOLVER solver)
 LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 {
 	LIS_MATRIX A;
-	LIS_PRECON M;
-	LIS_VECTOR b,x;
+	LIS_VECTOR x;
 	LIS_VECTOR r, rtld, mr, p, ap, map, t, mt_old, mt, amt, y, u, w, z;
 	LIS_QUAD_PTR alpha, beta, rho, rho_old;
 	LIS_QUAD_PTR qsi, eta, one;
 	LIS_QUAD_PTR tmp, tmpdot[5];
-	LIS_REAL   bnrm2, nrm2, tol;
-	LIS_INT iter,maxiter,n,output,conv;
-	double times,ptimes;
+	LIS_REAL bnrm2, nrm2, tol;
+	LIS_INT iter,maxiter,output,conv;
+	double time,ptime;
 
 	LIS_DEBUG_FUNC_IN;
 
 	A       = solver->A;
-	M       = solver->precon;
-	b       = solver->b;
 	x       = solver->x;
-	n       = A->n;
 	maxiter = solver->options[LIS_OPTIONS_MAXITER];
 	output  = solver->options[LIS_OPTIONS_OUTPUT];
 	conv    = solver->options[LIS_OPTIONS_CONV_COND];
-	ptimes  = 0.0;
+	ptime   = 0.0;
 
 	rtld    = solver->work[0];
 	r       = solver->work[1];
@@ -1625,10 +1604,10 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 
 	lis_solver_set_shadowresidual(solver,r,p);
 
-	LIS_MATVECT(A,p,rtld);
-	times = lis_wtime();
+	lis_matvect(A,p,rtld);
+	time = lis_wtime();
 	lis_psolve(solver, r, p);
-	ptimes += lis_wtime()-times;
+	ptime += lis_wtime()-time;
 	lis_vector_dotex_mmm(rtld,p,&rho_old);
 	lis_vector_set_allex_nm(0.0,t);
 	lis_vector_set_allex_nm(0.0,w);
@@ -1640,10 +1619,10 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 		/*   ap(k-1) = A * p(k-1)         */
 		/*   map(k-1)= M^-1 * ap(k-1)     */
 		/*   tmpdot0 = <map(k-1),rtld(0)> */
-		LIS_MATVEC(A,p,ap);
-		times = lis_wtime();
+		lis_matvec(A,p,ap);
+		time = lis_wtime();
 		lis_psolve(solver, ap, map);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,map,&tmpdot[0]);
 
 		if( tmpdot[0].hi[0]==0.0 && tmpdot[0].lo[0] )
@@ -1671,7 +1650,7 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 		{
 			if( output )
 			{
-				if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+				if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 				if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 			}
 
@@ -1680,7 +1659,7 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
@@ -1696,7 +1675,7 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 		/*   qsi     = (tmpdot0*tmpdot1-tmpdot2*tmpdot3) / tmp        */
 		/*   eta     = (tmpdot4*tmpdot2-tmpdot3*tmpdot1) / tmp        */
 		lis_vector_axpyzex_mmmm(alpha,map,mr,mt);
-		LIS_MATVEC(A,mt,amt);
+		lis_matvec(A,mt,amt);
 		lis_vector_dotex_mmm(y,y,&tmpdot[0]);
 		lis_vector_dotex_mmm(amt,t,&tmpdot[1]);
 		lis_vector_dotex_mmm(y,t,&tmpdot[2]);
@@ -1753,7 +1732,7 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 		lis_solver_get_residual[conv](r,solver,&nrm2);
 		if( output )
 		{
-			if( output & LIS_PRINT_MEM ) solver->residual[iter] = nrm2;
+			if( output & LIS_PRINT_MEM ) solver->rhistory[iter] = nrm2;
 			if( output & LIS_PRINT_OUT && A->my_rank==0 ) lis_print_rhistory(iter,nrm2);
 		}
 
@@ -1762,16 +1741,16 @@ LIS_INT lis_gpbicr_quad(LIS_SOLVER solver)
 			solver->retcode    = LIS_SUCCESS;
 			solver->iter       = iter;
 			solver->resid      = nrm2;
-			solver->ptimes     = ptimes;
+			solver->ptime      = ptime;
 			LIS_DEBUG_FUNC_OUT;
 			return LIS_SUCCESS;
 		}
 
 		/*   mr(k)   = M^-1 * r(k)                                          */
 		/*   rho(k)  = <rtld,mr(k)>                                         */
-		times = lis_wtime();
+		time = lis_wtime();
 		lis_psolve(solver, r, mr);
-		ptimes += lis_wtime()-times;
+		ptime += lis_wtime()-time;
 		lis_vector_dotex_mmm(rtld,mr,&rho);
 		if( rho.hi[0]==0.0 && rho.lo[0]==0.0 )
 		{
